package org.uclic.algebra;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import javax.swing.tree.*;
import org.jdom.Element;

/*
 * A node in a term tree.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public abstract class Term extends DefaultMutableTreeNode implements MutableTreeNode {
        
    /*
     * Every term belongs to a particular algebra.
     */
    protected EventAlgebra algebra;

    public final static int LIST_TYPE = 0; 
    public final static int TEXT_TYPE = 1; 
    public final static String IDREF = "ref";

    public final static int EVENT_DELETED = 0;
    public final static int STATE_DELETED = 1;
    public final static int CLASS_DELETED = 2;

    public Term(EventAlgebra a) {
	super();
	algebra = a;
    }

    public Term getSubterm(int i) {
	return (Term) getChildAt(i);
    }

    public int termType() {
	return LIST_TYPE;
    }

    public abstract String[] terms();
    public abstract Term buildTerm(int i);

    /**
     * Update the expression to reflect the deletion of an event.
     * @param i The index of the deleted event.
     */
    /*
    public boolean changeToAlgebra(int code, int e) {
	
	Enumeration kids = children();
	Term kid;
	boolean change = false;

	while(kids.hasMoreElements()) {
	    kid = (Term) kids.nextElement();
	    change = change || kid.changeToAlgebra(code, e);
	}

	return change;
    }
    */

    public int length() {
	int n = getChildCount();
	int l = 0;
	for (int i = 0; i < n; i++) {
	    l = getSubterm(i).length() + l;
	}
	return l;
    }

    public abstract Term copyTerm(EventAlgebra a);
    public abstract String toString();
    public abstract String termString();
    protected abstract String elementName();


    public void setAlgebra(EventAlgebra a) {
	algebra = a;
	Enumeration kids = children();
	while(kids.hasMoreElements()) {
	    Term kid = (Term) kids.nextElement();
	    kid.setAlgebra(algebra);
	}
    }
    
    public void deleteEvent(int i) {
	Enumeration kids = children();
	while(kids.hasMoreElements()) {
	    Term kid = (Term) kids.nextElement();
	    kid.deleteEvent(i);
	}
    }

    public void deleteState(int i) {
	Enumeration kids = children();
	while(kids.hasMoreElements()) {
	    Term kid = (Term) kids.nextElement();
	    kid.deleteState(i);
	}
    }

    public void deleteSuperstate(int i) {
	Enumeration kids = children();
	while(kids.hasMoreElements()) {
	    Term kid = (Term) kids.nextElement();
	    kid.deleteSuperstate(i);
	}
    }

    public void deleteSign(int i) {
	Enumeration kids = children();
	while(kids.hasMoreElements()) {
	    Term kid = (Term) kids.nextElement();
	    kid.deleteSign(i);
	}
    }


    public Element toXML() {

	int n = getChildCount();
	Element e = new Element(elementName());
	for (int i = 0; i < n; i++) {
	    e.addContent(getSubterm(i).toXML());
	}
	return e;
    }

}
