package org.uclic.algebra.event;

import java.util.List;
import java.util.NoSuchElementException;
import org.jdom.Element;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.matrix.Matrix;
import org.uclic.matrix.WrongSize;

/**
 * The product of several events.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class EventProduct extends EventSeries {
    
    public final static String STRING      = ".";
    public final static String NODE_STRING = "Product";

    public EventProduct(EventTerm e, EventAlgebra a) {
	super(e, a);
    }

    public EventProduct(EventAlgebra a) {
	super(a);
    }

    protected Matrix evaluate(Matrix a, Matrix b) throws WrongSize {
	return a.multiply(b);
    }
    
    public String termString(String s, String t) {
	return s + STRING + t;
    }

    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {	
	return FUI_XML.EPROD_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {

	int n = operands();

	EventTerm e = (EventTerm) operand(0).copyTerm(a);
	EventProduct p = new EventProduct(e, a);

	for (int i = 1; i < n; i++) {
	    EventTerm e2 = (EventTerm) operand(i).copyTerm(a);
	    p.add(e2);	    
	}

	return p;
    }
}
