package org.uclic.algebra.event;

import java.util.NoSuchElementException;
import org.jdom.Element;
import org.uclic.algebra.EvaluationException;
import org.uclic.algebra.EventAlgebra;
import org.uclic.algebra.Term;
import org.uclic.matrix.Matrix;
import org.uclic.matrix.WrongSize;

/**
 * An event series in an event algebra.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public abstract class EventSeries extends EventTerm {
   
    public EventSeries(EventTerm e, EventAlgebra a) {
	super(a);
	add(e);
    }

    public EventSeries(EventAlgebra a) {
	super(a);
	add(new UIEvent(a));
	add(new UIEvent(a));
    }

    public Matrix evaluate() throws EvaluationException {

	EventTerm e = (EventTerm) getSubterm(0);
	Matrix m = e.evaluate();
	int n = getChildCount();

	try {	
	    for (int i = 1; i < n; i++) {
		e = (EventTerm) getSubterm(i);
		m = evaluate(m, e.evaluate());
	    }	    
	} catch (WrongSize ws) {
	    throw new EvaluationException("Event series " + toString() + " has incompatible elements.");
	}
	
	return m;
    }

    protected abstract Matrix evaluate(Matrix a, Matrix b) throws WrongSize;

    public int operands() {
	return getChildCount();
    }

    public EventTerm operand(int i) {
	return (EventTerm) getSubterm(i);
    }

    public int events() {
	int e = 0;
	int n = getChildCount();
	EventTerm t;
	for (int i = 0; i < n; i++) {
	    t = (EventTerm) getSubterm(i);
	    e = e + t.events();
	}
	return e;
    }

    public String termString() {
	String s = getSubterm(0).termString();
	int n = getChildCount();
	for (int i = 1; i < n; i++) {
	    s = termString(s, getSubterm(i).termString());
	}	 	
	return s;
    }

    public abstract String termString(String s, String t);


    public EventIterator eventIterator() {
	return new EventSeriesIterator();
    }

    class EventSeriesIterator implements EventIterator {

	private EventIterator itr;
	private int kid;

	public EventSeriesIterator() {
	    kid = 0;
	    EventTerm e = (EventTerm) getSubterm(0);
	    itr = e.eventIterator();
	}

	public boolean hasNext() {
	    return itr.hasNext();
	}

	public AtomicEvent next() throws NoSuchElementException {

	    AtomicEvent n = itr.next();
		
	    if (!itr.hasNext()) {
		kid++;
		if (kid < getChildCount()) {
		    EventTerm t = (EventTerm) getSubterm(kid);
		    itr = t.eventIterator();
		}
	    }
	    return n;
	}
    }
}

