package org.uclic.algebra.event;

import org.jdom.Element;
import org.uclic.algebra.*;
import org.uclic.matrix.Matrix;
import org.uclic.fui.FUIException;
import org.uclic.fui.SignFSM;

/*
 * A event algebra term representing an event.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public abstract class EventTerm extends Term {

    private final static int PREDEFINED = 5;

    public EventTerm(EventAlgebra a) {
	super(a);
    }

    /**
     * The event matrix for this expression.
     * @return An event matrix.
     */
    public abstract Matrix evaluate() throws EvaluationException;

    public abstract int events();
    public abstract EventIterator eventIterator();


    public String[] terms() {
	
	SignFSM m = algebra.getModel();
	int e = m.events().length;
	String[] terms = new String[e + PREDEFINED];

	terms[0] = Id.STRING;
	terms[1] = EventProduct.NODE_STRING;
	terms[2] = EventSum.NODE_STRING;
	terms[3] = Power.NODE_STRING;
	terms[4] = Goto.NODE_STRING;

	for (int i = 0; i < e; i++) {
	    try {
		terms[i + PREDEFINED] = m.event(i);
	    } catch (FUIException me) {}
	}
       
	return terms;
    }
    
    public Term buildTerm(int i) {
	
	if (i < PREDEFINED) {
	    switch (i) {

	    default:
		return new Id(algebra);

	    case 1:
		return new EventProduct(algebra);

	    case 2:
		return new EventSum(algebra);

	    case 3:
		return new Power(algebra);
		
	    case 4:
		return new Goto(algebra);
	    }
	} else {
	    return new UIEvent(i - PREDEFINED, algebra);
	}
    }

}
