package org.uclic.algebra.event;

import java.util.List;
import org.jdom.Element;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.state.*;
import org.uclic.matrix.Matrix;
import org.uclic.matrix.RowVector;

/**
 * An user interface event in an event algebra.
 *
 * @version 0.3.2
 * @author Jeremy Gow
 */
public class Goto extends AtomicEvent {
   
    public final static String PRE_STRING  = "GOTO(";
    public final static String POST_STRING = ")";
    public final static String NODE_STRING = "Goto";

    public Goto(StateTerm s, EventAlgebra a) {
	super(a);
	add(s);
    }

    public Goto(EventAlgebra a) {
	this(new UIState(a), a);
    }
    
    public Matrix evaluate() throws EvaluationException {
	StateTerm s = (StateTerm) getSubterm(0);
	RowVector v = s.evaluate();
	return v.stack(v.cols());
    }

    public String termString() {
	return PRE_STRING + getSubterm(0).termString() + POST_STRING;
    }
   
    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {
	return FUI_XML.GOTO_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {
	
	StateTerm s = (StateTerm) getSubterm(0);
	StateTerm s2 = (StateTerm) s.copyTerm(a);
	return new Goto(s2, a);
    }   
}

