package org.uclic.algebra.event;

import java.util.List;
import org.jdom.Element;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.matrix.Matrix;

/**
 * An user interface event in an event algebra.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class UIEvent extends AtomicEvent {
   
    private int index;

    public final static String PRE_STRING  = "[";
    public final static String POST_STRING = "]";
    public final static String UNKNOWN_STRING = "?";

    public UIEvent(int i, EventAlgebra a) {
	super(a);
	index = i;
    }

    public UIEvent(EventAlgebra a) {
	this(-1, a);
    }

    public int eventIndex() {
	return index;
    }
    
    public Matrix evaluate() throws EvaluationException {
	return algebra.eventMatrix(index);
    }
    
    /*
    public boolean changeToAlgebra(int code, int i) {

	boolean change = false;

	if (code == EVENT_DELETED) {
	    if (i < index) {
		index--;
	    } else if (i == index) {
		index = -1;
		change = true;
	    }
	}

	return change;
    }
    */

    public void deleteEvent(int i) {
	 if (i < index) {
		index--;
	 } else if (i == index) {
	     index = -1;
	 }
    }

    public String termString() {
	try {
	    return PRE_STRING + algebra.getModel().event(index) + POST_STRING;
	} catch (FUIException e) {
	    return UNKNOWN_STRING;
	}
    }
   
    public String toString() {
	return termString();
    }

    public String elementName() {
	return FUI_XML.UIEVENT_ELEM;
    }

    public Element toXML() {
	
	Element e = new Element(elementName());
	try {
	    e.setAttribute(IDREF, algebra.getModel().event(index));
	} catch (FUIException me) {}

	return e;
    }
   
    public Term copyTerm(EventAlgebra a) {
	return new UIEvent(index, a);
    }
}

