package org.uclic.algebra.property;

import java.util.List;
import org.jdom.Element;
import org.uclic.algebra.*;
import org.uclic.fui.*;

/**
 * The negation of a property.
 *
 * @version 0.3.2
 * @author Jeremy Gow
 */
public class Not extends PropertyTerm {
    
    public final static String PRE_STRING  = "not(";
    public final static String POST_STRING = ")";
    public final static String NODE_STRING = "not";

    public Not(PropertyTerm t, EventAlgebra a) {
	super(a);
	add(t);
    }

    public Not(EventAlgebra a) {
	this(new EventEquality(a), a);
    }

    public PropertyTerm getProperty() {
	return (PropertyTerm) getChildAt(0);
    }

    public boolean[] evaluate() throws EvaluationException {
	
	boolean[] eval = getProperty().evaluate();

	for (int i = 0; i < eval.length; i++) {
	    eval[i] = !eval[i];
	}

	return eval;
    }

    public int length() {
	return getProperty().length();
    }

    public String termString() {
	return PRE_STRING + getProperty().termString() + POST_STRING;
    }

    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {
	return FUI_XML.NOT_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {

	PropertyTerm p = (PropertyTerm) getProperty().copyTerm(a);
       	return new Not(p, a);
    }

 }

