package org.uclic.algebra.property;

import java.util.List;
import org.jdom.Element;
import org.uclic.algebra.*;
import org.uclic.algebra.state.StateTerm;
import org.uclic.algebra.state.UIState;
import org.uclic.matrix.*;

/**
 * The equality of two states.
 *
 * @version 0.3.2
 * @author Jeremy Gow
 */
public class StateEquality extends Equality {
    
    public static final String NODE_STRING = "Equals (states)";

    public StateEquality(StateTerm s1, StateTerm s2, EventAlgebra a) {
	super(s1, s2, a);
    }

    public StateEquality(EventAlgebra a) {
	super(new UIState(a), new UIState(a), a);
    }

    public boolean[] evaluate() throws EvaluationException {
	
	StateTerm lhs = (StateTerm) getSubterm(0);
	StateTerm rhs = (StateTerm) getSubterm(1);

	return lhs.evaluate().equalColumns(rhs.evaluate());
    }

    public String toString() {
	return NODE_STRING;
    }

    public Term copyTerm(EventAlgebra a) {

	StateTerm lhs = (StateTerm) getSubterm(0);
	StateTerm rhs = (StateTerm) getSubterm(1);
	StateTerm e1 = (StateTerm) lhs.copyTerm(a);
	StateTerm e2 = (StateTerm) rhs.copyTerm(a);
       	return new StateEquality(e1, e2, a);
    }
}

