package org.uclic.algebra.property;

import java.util.List;
import org.jdom.Element;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.event.*;
import org.uclic.matrix.*;

/**
 * The property that an individual event can be undone.
 *
 * @version 0.3
 * @author Jeremy Gow
 */
public class Undo extends PropertyTerm {
    
    public static final String PRE_STRING  = "undo(";
    public static final String POST_STRING = ")";
    public static final String NODE_STRING = "undo";

    public Undo(EventTerm e, EventAlgebra a) {
	super(a);
	add(e);
    }

    public Undo(EventAlgebra a) {
	this(new UIEvent(a), a);
    }
   
    public boolean[] evaluate() throws EvaluationException {

	EventTerm e = (EventTerm) getSubterm(0);
	Matrix m = e.evaluate();
	boolean ns = !m.singular();
	int r = m.rows();

	boolean[] eval = new boolean[r];
	for (int i = 0; i < r; i++) {
	    eval[i] = ns;
	}

	return eval;
    }

    public String termString() {
	return PRE_STRING + getSubterm(0).termString() + POST_STRING;
    }

    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {
	return FUI_XML.UNDO_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {

	EventTerm e = (EventTerm) getSubterm(0);
	EventTerm e2 = (EventTerm) e.copyTerm(a);
       	return new Undo(e2, a);
    }
}

