package org.uclic.algebra.state;


import java.util.List;
import java.util.NoSuchElementException;
import org.jdom.Element;
import org.uclic.algebra.*;
import org.uclic.fui.*;
import org.uclic.matrix.RowVector;
import org.uclic.matrix.WrongSize;

/**
 * The sum of several states, i.e. we're in one of the states.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class StateSum extends StateTerm {
    
    public final static String STRING      = " + ";
    public final static String NODE_STRING = "Sum";

    public StateSum(StateTerm s, EventAlgebra a) {
	super(a);
	add(s);
    }

    public StateSum(EventAlgebra a) {
	super(a);
	add(new UIState(a));
	add(new UIState(a));
    }

    public RowVector evaluate() throws EvaluationException {

	StateTerm s = (StateTerm) getSubterm(0);
	RowVector m = s.evaluate();
	int n = getChildCount();

	try {	
	    for (int i = 1; i < n; i++) {
		s = (StateTerm) getSubterm(i);
		m = m.addVector(s.evaluate());
	    }	    
	} catch (WrongSize e) {
	    throw new EvaluationException("State sum " + toString() + " has incompatible vectors.");
	}
	
	return m;
    }

    public int operands() {
	return getChildCount();
    }

    public StateTerm operand(int i) {
	return (StateTerm) getSubterm(i);
    }

    public String termString() {

	String s = getSubterm(0).termString();
	int n = getChildCount();

	for (int i = 1; i < n; i++) {
	    s = s + STRING + getSubterm(i).termString();
	}	 
	
	return s;
    }

    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {
	return FUI_XML.SSUM_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {

	int n = operands();

	StateTerm e = (StateTerm) operand(0).copyTerm(a);
	StateSum s = new StateSum(e, a);

	for (int i = 1; i < n; i++) {
	    StateTerm e2 = (StateTerm) operand(i).copyTerm(a);
	    s.add(e2);	    
	}

	return s;
    }
}



