package org.uclic.algebra.state;

import org.jdom.Element;
import org.uclic.algebra.*;
import org.uclic.algebra.event.*;
import org.uclic.fui.*;
import org.uclic.matrix.RowVector;

/*
 * A event algebra term representing an state.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public abstract class StateTerm extends Term {

    private final static int PREDEFINED = 2;

    public StateTerm(EventAlgebra a) {
	super(a);
    }

    /**
     * The row vector for this state.
     * @return A row vector.
     */
    public abstract RowVector evaluate() throws EvaluationException;

    /*
     * Returns an array of strings representing possible instances of
     * this class.
     */
    public String[] terms() {
	
	SignFSM m = algebra.getModel();
	int s = m.states().length;
	int c = m.superstates().length;
	int g = m.signs().length;
	String[] terms = new String[s + c + g + PREDEFINED];

	terms[0] = Transition.NODE_STRING;
	terms[1] = StateSum.NODE_STRING;

	// Add state strings
	for (int i = 0; i < s; i++) {
	    try {
		terms[PREDEFINED + i] = m.state(i);
	    } catch (FUIException e) {}
	}

	// Add class strings
	for (int i = 0; i < c; i++) {
	    try {
		terms[PREDEFINED + s + i] = m.superstate(i);
	    } catch (FUIException e) {}
	}

	// Add class strings
	for (int i = 0; i < g; i++) {
	    try {
		terms[PREDEFINED + s + c + i] = m.sign(i);
	    } catch (FUIException e) {}
	}
       
	return terms;
    }    

    public Term buildTerm(int i) {
	
	Term term = null;

	if (i < PREDEFINED) {
	    switch (i) {

	    default:
		return new Transition(algebra);

	    case 1:
		return new StateSum(algebra);
	    }
	} else {
	    
	    int s = algebra.getModel().states().length;
	    int c = algebra.getModel().superstates().length;

	    if (i < PREDEFINED + s) {
		return new UIState(i - PREDEFINED, algebra);
		
	    } else if (i < PREDEFINED + s + c) {
		return new UISuper(i - (PREDEFINED + s), algebra);	
	    
	    } else {
		return new UISign(i - (PREDEFINED + s + c), algebra);
	    }
	}
    }
}
