package org.uclic.algebra.state;

import java.util.List;
import java.util.NoSuchElementException;
import org.jdom.Element;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.event.*;
import org.uclic.algebra.state.*;
import org.uclic.matrix.*;

/**
 * The product of several events.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class Transition extends StateTerm {
    
    public final static String STRING      = ".";
    public final static String NODE_STRING = "Product";

    public Transition(StateTerm s, EventTerm e, EventAlgebra a) {
	super(a);
	add(s);
	add(e);
    }

    public Transition(EventAlgebra a) {
	this(new UIState(a), new UIEvent(a), a);
    }

    public RowVector evaluate() throws EvaluationException {

	StateTerm state = (StateTerm) getSubterm(0);
	EventTerm event = (EventTerm) getSubterm(1);

	try {
	    return state.evaluate().transform(event.evaluate());
	} catch (WrongSize e) {
	    throw new EvaluationException(e.getMessage());
	}

    }

    public String termString() {
	return getSubterm(0).termString() + STRING + getSubterm(1).termString();
    }

    public String toString() {
	return NODE_STRING;
    }

    public String elementName() {
	return FUI_XML.SPROD_ELEM;
    }

    public Term copyTerm(EventAlgebra a) {
	
	StateTerm s = (StateTerm) getSubterm(0);
	EventTerm e = (EventTerm) getSubterm(1);
	StateTerm s2 = (StateTerm) s.copyTerm(a);
	EventTerm e2 = (EventTerm) e.copyTerm(a);
	
	return new Transition(s2, e2, a);
    }
    
}

