package org.uclic.algebra.state;

import java.util.List;
import org.jdom.Element;
import org.uclic.matrix.RowVector;
import org.uclic.algebra.*;
import org.uclic.fui.*;

/**
 * An algebraic element corresponding to the states in which a given
 * sign is displayed.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class UISign extends AtomicState {
   
    private int index;

    public final static String PRE_STRING     = "<";
    public final static String POST_STRING    = ">";
    public final static String UNKNOWN_STRING = "?";

    public UISign(int i, EventAlgebra a) {
	super(a);
	index = i;
    }

    public UISign(EventAlgebra a) {
	this(-1, a);
    }
    
    public RowVector evaluate() throws EvaluationException {
	return algebra.signVector(index);
    }

    /*
    public boolean changeToAlgebra(int code, int i) {

	boolean change = false;

	if (code == CLASS_DELETED) {
	    if (i < index) {
		index--;
	    } else if (i == index) {
		index = -1;
		change = true;
	    }
	}

	return change;
    }
    */

    public void deleteSign(int i) {
	 if (i < index) {
		index--;
	 } else if (i == index) {
	     index = -1;
	 }
    }    
    
    public String termString() {
	try {
	    return PRE_STRING + algebra.getModel().sign(index) + POST_STRING;
	} catch (FUIException e) {
	    return UNKNOWN_STRING;
	}
    }
   
    public String toString() {
	return termString();
    }

    public Element toXML() {
	
	Element e = new Element(elementName());
	try {
	    e.setAttribute(IDREF, algebra.getModel().superstate(index));
	} catch (FUIException me) {}

	return e;
    }

    public String elementName() {
	return FUI_XML.SIGN_ELEM;
    }
    
    public Term copyTerm(EventAlgebra a) {
	return new UISign(index, a);
    }
}
