package org.uclic.algebra.state;

import java.util.List;
import org.jdom.Element;
import org.uclic.matrix.RowVector;
import org.uclic.algebra.*;
import org.uclic.fui.*;

/**
 * An algebraic element corresponding to an interface state.
 *
 * @version 0.4
 * @author Jeremy Gow
 */
public class UIState extends AtomicState {
   
    private int index;

    public final static String UNKNOWN_STRING = "?";

    public UIState(int i, EventAlgebra a) {
	super(a);
	index = i;
    }

    public UIState(EventAlgebra a) {
	this(-1, a);
    }
    
    public RowVector evaluate() throws EvaluationException {
	return algebra.stateVector(index);
    }

    /*
    public boolean changeToAlgebra (int code, int i) {

	boolean change = false;

	if (code == STATE_DELETED) {
	    if (i < index) {
		index--;
	    } else if (i == index) {
		index = -1;
		change = true;
	    }
	}

	return change;
    }
    */

    public void deleteState(int i) {
	 if (i < index) {
		index--;
	 } else if (i == index) {
	     index = -1;
	 }
    }
    
   public String termString() {
	try {
	    return algebra.getModel().state(index);
	} catch (FUIException e) {
	    return UNKNOWN_STRING;
	}
    }
   
    public String toString() {
	return termString();
    }

    protected String elementName() {
	return FUI_XML.UISTATE_ELEM;
    }

    public Element toXML() {
       
	Element e = new Element(elementName());
	try {
	    e.setAttribute(IDREF, algebra.getModel().state(index));
	} catch (FUIException me) {}

	return e;
    }

    public Term copyTerm(EventAlgebra a) {
	return new UIState(index, a);
    }
}

