package org.uclic.maui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.beans.PropertyVetoException;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;
import java.util.jar.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.uclic.fui.FUIDesign;
import org.uclic.fui.FUIException;
import org.uclic.maui.edit.Editor;
import org.uclic.maui.analyse.Analysis;
import org.uclic.maui.sim.*;
//import org.uclic.maui.skins.*;
import org.uclic.maui.stats.*;
import org.uclic.maui.plugin.*;
import org.uclic.maui.plugin.MAUIPluginAdapter;

/**
 * The MAUI application.
 *
 * @author Jeremy Gow
 * @version 0.4
 *
 * Move code for title box etc into MAUIToolBar.
 */
public class MAUI extends MAUIFrame implements ActionListener, ChangeListener {
    
    private static String     home;
    private static Image      icon;
    private static String     prefs;
    private static Properties props;
        
    private FUIDesign    design;

    private JTabbedPane  tabs;
    private MAUITab      selected;
    private JTextField   title;
    private JComboBox    initial;
    private JLabel       status;
    private FileOperator operator;
    private JFrame       help;
    private Vector       plugins;

    private JTable       table1;
    private JTable       table2;

    // Dimensions
    public final static int XDIM        = 1000;  
    public final static int YDIM        = 800;  
    public final static int STATUS_YDIM = 20;  
    

    // The Disclaimer Message
    public final static String DMR1 = "MAUI comes with ABSOLUTELY NO WARRANTY\n";
    public final static String DMR2 = "This is free software, and you are welcome to redistribute it\n";
    public final static String DMR3 = "under certain conditions.  Please view the accompanying file\n";
    public final static String DMR4 = "'COPYING' for more information.";
    public final static String DISCLAIMER = DMR1 + DMR2 + DMR3 + DMR4;


    public final static String VERSION = "0.4";
    public final static String COPYRIGHT = "Copyleft 2004 Jeremy Gow\n";
    public final static String INITIAL_STATUS = "Welcome to MAUI: Mathematical Analysis of User Interfaces";

    // The About Message
    public final static String MAUI = "MAUI: Matrix Analysis of User Interfaces\nVersion " + VERSION + "\n\n";
    public final static String ADDRESS1 = "UCLIC (UCL Interaction Centre)\n";
    public final static String ADDRESS2 = "University College London, UK\n";
    public final static String ADDRESS3 = "j.gow@ucl.ac.uk\nwww.uclic.ucl.ac.uk/usr/jgow/maui/\n\n";
    public final static String ABOUT_TEXT = MAUI + COPYRIGHT + ADDRESS1 + ADDRESS2 + ADDRESS3 + DISCLAIMER;

    // The Help Message
    public final static String HLP1 = "MAUI does not currently have a help option,\n";
    public final static String HLP2 = "but some helpful references can be found at:\n";
    public final static String HLP3 = "http://www.uclic.ucl.ac.uk/usr/jgow/maui/";
    public final static String HELP_TEXT = HLP1 + HLP2 + HLP3;
    
    // Interface Text
    public final static String WINDOW_TITLE = "MAUI: ";
    public final static String MODEL_TITLE  = "Name: ";
    public final static String INIT_TITLE   = "Start: ";

    // Edit codes
    public final static int NEW_DESIGN  = 0;
    public final static int NEW_TITLE   = 1;
    public final static int MODEL_EDIT  = 2;
    public final static int RENAME_EDIT = 3;
    public final static int SPEC_EDIT   = 4;

    // Distribution files
    public final static String ICONS      = "icons";
    public final static String DOC_DIR    = "doc";
    public final static String MODEL_DIR  = "models";
    public final static String PLUGIN_DIR = "plugins";
    public final static String HELP_DIR   = "help";
    public final static String HELP_FILE  = "index.html";
    public final static String ICON_FILE  = "maui.gif";
    public final static String PREFS_FILE = ".maui_prefs";
    public final static String DTD        = "fui.dtd";

    public final static String CLASS_SUFFIX = ".class";
    public final static String JAR_SUFFIX   = ".jar";

    // Properties
    public final static String DIR_PROP = "maui.dir";
    public final static String CURRENT_PROP = "maui.current";
    public final static String PREV1_PROP = "maui.prev1";
    public final static String PREV2_PROP = "maui.prev2";
    public final static String PREV3_PROP = "maui.prev3";
    public final static String PREV4_PROP = "maui.prev4";

    public static void main(String[] args) {
        if (args.length > 0) {
            try {
		/*
		UIManager.LookAndFeelInfo[] l = UIManager.getInstalledLookAndFeels();
		System.out.print("\n\n");
		for (int i = 0; i < l.length; i++) {
		    System.out.println(l[i].toString() + "\n");
		}
		UIManager.installLookAndFeel(l[0]);
		*/
	    } catch (Exception e) {
		System.out.println("MAUI: Top-level error.");
		e.printStackTrace();
	    }
	    
	    String sep = System.getProperty("file.separator");
	    String icn = args[0] + sep + ICONS + sep + ICON_FILE;    
	    Image i = Toolkit.getDefaultToolkit().getImage(icn);
            new MAUI(args[0], i);
        }
    }
    
    public MAUI(String h, Image k) {
        super(WINDOW_TITLE, k);
        home = h;
	icon = k;
	
        // Load preferences.
	String sep = System.getProperty("file.separator");
	prefs = home + sep + PREFS_FILE;
        props = new Properties();
        loadPrefs();
	loadPlugins();

	// Default design.
	design = new FUIDesign();
    
        // Configure filechooser
        operator = new FileOperator(this);
        File dir = new File(props.getProperty(DIR_PROP));
        if (!dir.exists()) {
            dir = new File(home);
        }
	try {
	    operator.setCurrentDirectory(dir);
	} catch (ArrayIndexOutOfBoundsException e) {
	    // Sometimes an exception is generated by the file
	    // operator here - I have no idea why it happens, or why
	    // most of the time it doesn't...
	} 

        // Read in last design
        File current = new File(props.getProperty(CURRENT_PROP));
        if (current.exists()) {
            operator.setSelectedFile(current);
            try {
		design = operator.readModel();
	    } catch (Exception e) {
		//System.out.println("\nMAUI - Error reading model:\n" + e.getMessage() + "\n");
		//e.printStackTrace();		
	    }
        }
		
        // Build window
        buildMAUI();
	setVisible(true);
    }

    public int xDim() {return XDIM;}
    public int yDim() {return YDIM;}

    private void buildMAUI() {
	// Build Tabs and Title
	tabs = new JTabbedPane();
	title = new JTextField(30);
	title.setMaximumSize(new Dimension(200,30));
	initial = new JComboBox();
	initial.setBackground(Color.WHITE);
	initial.setMaximumSize(new Dimension(100,30));
	status = new JLabel(" " + INITIAL_STATUS);
	status.setPreferredSize(new Dimension((int) (XDIM * 0.95), STATUS_YDIM));
	status.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
       
	buildStats();

	addTabsAndTitle();
	
	// Add listeners
	tabs.addChangeListener(this);
	title.addActionListener(this);
	title.setActionCommand(MODEL_TITLE);
	initial.addActionListener(this);
	initial.setActionCommand(INIT_TITLE);

      	JPanel pane = new JPanel();
	pane.setLayout(new BorderLayout());
	pane.add(new MAUIToolBar(this, title, initial, table1, table2), BorderLayout.PAGE_START);
	pane.add(tabs, BorderLayout.CENTER);
	pane.add(status, BorderLayout.PAGE_END);
        setContentPane(pane);

        // Add Menu Bar
        setJMenuBar(new MAUIMenu(this));
    }
    
    private void buildStats() {
      	table1 = statTable("States", "Events");
	table2 = statTable("Superst.", "Signs");
    }

    private JTable statTable(String s1, String s2) {

	JTable table = new JTable(2, 2);
	table.setValueAt(s1, 0, 0);
	table.setValueAt(s2, 1, 0);
	table.setCellSelectionEnabled(false);

	//table.setRowHeight(10);
	TableColumnModel m = table.getColumnModel();
	m.getColumn(0).setPreferredWidth(50);
	m.getColumn(1).setPreferredWidth(30);
	m.getColumn(0).setMaxWidth(50);
	m.getColumn(1).setMaxWidth(30);

	return table;
    }

    private void addTabsAndTitle() {
	setTitle();
	setInitial();
	setStats();
	//addTab(new Statistics(this));
        addTab(new Simulation(this));
        addTab(new Editor(this));
	//addTab(new Skins(this));
	addTab(new Analysis(this));
	addTab(new XMLFrame(this));
	
	Iterator itr = plugins.iterator();
	while (itr.hasNext()) {
	    MAUIPlugin p = (MAUIPlugin) itr.next();
	    PluginTab pt = new PluginTab(this, p);
	    tabs.addTab(pt.title(), pt.icon(), pt, pt.tip());
	}
	alertSelectedTab();
    }

    public void setTitle() {
        setTitle(WINDOW_TITLE + design.name());
	title.setText(design.name());
    }

    public void setInitial() {
	initial.setModel(new DefaultComboBoxModel(design.model().states()));
	initial.setSelectedIndex(design.model().initialStateIndex());
    }

    public void setStats() {

	Integer s = new Integer(design.model().states().length);
	Integer e = new Integer(design.model().events().length);
	Integer p = new Integer(design.model().superstates().length);
	Integer g = new Integer(design.model().signs().length);
	
	table1.setValueAt(s, 0, 1);
	table1.setValueAt(e, 1, 1);
	table2.setValueAt(p, 0, 1);
	table2.setValueAt(g, 1, 1);
    }

    public void clearStatus() {
	status.setText("");
    }

    public void setStatus(String text) {
	status.setText(" " + text);
	status.repaint();
    }
    
    private void addTab(MAUITab tab) {
	tabs.addTab(tab.title(), tab.icon(), tab, tab.tip());
    }

    public void designChanged(int code) {

	switch (code) {

	case NEW_DESIGN:
	    setTitle();
	    setStats();
	    break;

	case NEW_TITLE:
	    setTitle();
	    break;

	case MODEL_EDIT:
	    setStats();
	    break;
	}

	int t = tabs.getTabCount();
	for (int i = 0; i < t; i++) {
	    MAUITab mt = (MAUITab) tabs.getComponentAt(i);
	    if (mt != selected) {
		mt.designChanged(code);
	    }
	}
    }

    
    // Global data

    public FUIDesign design() {
	return design;
    }

    public void setDesign(FUIDesign fd) {
	design = fd;
	buildMAUI();
    }

    public FileOperator operator() {
        return operator;	
    }

    public String home() {
	return home;
    }

    public Image icon() {
        return icon;	
    }

    public String iconPath(String file) {
	String sep = System.getProperty("file.separator");
	return home + sep + ICONS + sep + file;
    }

    // Display an error message
    public void error(String pStr) {
        JOptionPane.showMessageDialog(this, pStr, "MAUI Error",
                JOptionPane.ERROR_MESSAGE);
    }
    
    public void about() {
        JOptionPane.showMessageDialog(this,
                ABOUT_TEXT, WINDOW_TITLE + MAUIMenu.ABOUT_MAUI,
                JOptionPane.INFORMATION_MESSAGE, new ImageIcon(icon));
    }

    public void setHelp(boolean set) {
	
	//JOptionPane.showMessageDialog(this,
	//			      HELP_TEXT, WINDOW_TITLE + MAUIMenu.MAUI_HELP,
	//			      JOptionPane.INFORMATION_MESSAGE, new ImageIcon(icon));
	
	if (set) {
	    if (help == null) {
		help = new MAUIHelp(this, icon);
	    }
	} else {
	    help.setVisible(false);
	    help.dispose();
	    help = null;	    
	}
    }

    /**
     * Loading and Saving Preferences
     */
    
    public String pref(String pPref, String pDefault) {
        return props.getProperty(pPref, pDefault);
    }
    
    public void setPref(String pPref, String pDefault) {
        props.setProperty(pPref, pDefault);
    }
    
    private void loadPrefs() {
        File prf = new File(prefs);

        if (prf.exists()) {
            try {
                props.load(new FileInputStream(prf));
            } catch (IOException e) {}
        } else {
            props.setProperty(DIR_PROP, home);
            props.setProperty(CURRENT_PROP, "");
        }
    }
    
    private void savePrefs() {
        String header = "MAUI Preferences File";

        try {
            File dir = operator.getCurrentDirectory();

            props.setProperty(DIR_PROP, dir.getAbsolutePath());
            props.store(new FileOutputStream(prefs), header);
        } catch (IOException ex) {}
    }

    /*
     * Loading plugins.
     */
    private void loadPlugins() {
	
	URLClassLoader mauiLoader = (URLClassLoader) this.getClass().getClassLoader();

	plugins = new Vector();
	String sep = System.getProperty("file.separator");
	File plugDir = new File(home + sep + PLUGIN_DIR + sep);

	// Load .class & .jar files
	File[] files = plugDir.listFiles(new FileFilter() {
		public boolean accept(File f) {
		    String n = f.getName();
		    return n.endsWith(CLASS_SUFFIX) | n.endsWith(JAR_SUFFIX);
		}
	    });
	
	if (files != null) {

	    URL[] urls = new URL[files.length];
	    
	    for (int i = 0; i < files.length; i++) {
		try {
		    if (files[i].getName().endsWith(JAR_SUFFIX)) {
			urls[i] = new URL("jar:file:" + files[i].getAbsolutePath() + "!/");
		    } else {
			urls[i] = files[i].toURL();
		    }
		} catch (MalformedURLException e) {}
	    }
	    
	    URLClassLoader pluginLoader = new URLClassLoader(urls, mauiLoader);
	    
	    for (int i = 0; i < files.length; i++) {
		
		URLClassLoader loader = pluginLoader;
		String name = files[i].getName();
		String classname = "";
		
		try {
		    if (name.endsWith(JAR_SUFFIX)) {
			// .jar file
			JarURLConnection con = (JarURLConnection) urls[i].openConnection();
			Attributes att = con.getMainAttributes();
			
			if (att != null) {
			    classname = att.getValue(Attributes.Name.MAIN_CLASS);
			    
			    // Get Class-Path from .jar MANIFEST
			    String classpath =  att.getValue(Attributes.Name.CLASS_PATH);
			    Vector classpaths = new Vector();
			    System.out.println("Class-Path: " + classpath);
			    
			    while (classpath.indexOf(" ") != -1) {
				int j = classpath.indexOf(" ");
				classpaths.add(classpath.substring(0, j));
				classpath = classpath.substring(j + 1, classpath.length());
			    }
			    classpaths.add(classpath);
			    
			    URL[] cpURLs = new URL[classpaths.size()];
			    for (int k = 0; k < classpaths.size(); k++) {
				String cp = (String) classpaths.elementAt(k);
				cpURLs[k] = new URL("jar:file:" + cp + "!/");
			    }
			    loader = new URLClassLoader(cpURLs, pluginLoader);
			}
			
		    } else {
			// .class file
			int j = name.lastIndexOf(CLASS_SUFFIX);
			classname = name.substring(0, j);
		    }    
		    
		    System.out.println("About to load " + classname);
		    showurls(loader);
		    showurls((URLClassLoader) loader.getParent());
		    showurls((URLClassLoader) loader.getParent().getParent());
		    
		    Class c = loader.loadClass(classname);    
		    plugins.add((MAUIPlugin) c.newInstance());
		    
		} catch (Throwable t) {
		    // Do nothing if any kind of error when loading class
		    System.out.println("\n");
		    t.printStackTrace();
		}
	    }
	}
    }

    private void showurls(URLClassLoader l) {
	if (l != null) {
	    URL[] u = l.getURLs();
	    for (int k = 0; k < u.length; k++) {
		System.out.println("URL: " + u[k]);
	    }	       
	}
    }


    /*
     * Exit
     */
    public void exit() {
	if (help != null) {
	    setHelp(false);
	}
        setVisible(false);
        savePrefs();
        dispose();
    }   


    public void actionPerformed(ActionEvent e) {
	
        String com = e.getActionCommand();
	
	// FILE MENU
	if (com.equals(MAUIMenu.EXIT_APP)) {
	    exit();
	 
	} else if (com.equals(MAUIMenu.NEW_DESIGN)) {
	    setDesign(new FUIDesign());
	    designChanged(NEW_DESIGN);
	    
	// HELP MENU
	} else if (com.equals(MAUIMenu.ABOUT_MAUI)) {
	    about();

	} else if (com.equals(MAUIMenu.MAUI_HELP)) {
	    setHelp(true);
	    
	// EDIT MODEL NAME 
	} else if (com.equals(MODEL_TITLE)) {
	    try {
		design.setName(title.getText());
		designChanged(NEW_TITLE);
	    } catch (FUIException fe) {
		// Invalid name
		title.setText(design.name());
	    }
	} else if (com.equals(INIT_TITLE)) {

	    design.model().setInitialState(initial.getSelectedIndex());
	}
    }

    /**
     * Fired when a new tab is selected.
     *
     */
    public void stateChanged(ChangeEvent e) {
	alertSelectedTab();
    }

    private void alertSelectedTab() {
	selected = (MAUITab) tabs.getSelectedComponent();
	selected.tabSelected();
    }

    public MAUITab selected() {
	return selected;
    }    
}
