package org.uclic.maui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.tree.*;

/**
 * The MAUI help window.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class MAUIHelp extends MAUIFrame {

    private MAUI     maui;
    private HelpHTML menu;
    private HelpHTML main;

    private final static String TITLE = "MAUI Help";
    private final static int XDIM = 800;
    private final static int YDIM = 600;

    public MAUIHelp(MAUI m, Image icon) {
	super(TITLE, icon);
	maui = m;
	build();
	setVisible(true);
    }

    public int xDim() {return XDIM;}
    public int yDim() {return YDIM;}

    public void exit() {
	maui.setHelp(false);
    }

    public void build() {
	menu = new HelpHTML("contents.html");
	main = new HelpHTML("intro.html");
	menu.addHyperlinkListener(new Hyperactive(main));
	main.addHyperlinkListener(new Hyperactive(main));

	JScrollPane scroll1 = new JScrollPane(menu);
	JScrollPane scroll2 = new JScrollPane(main);
	scroll1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	scroll2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

	//scroll1.setPreferredSize(new Dimension(250, 145));
	int x1 = (int) (XDIM * 0.2);
	int x2 = (int) (XDIM * 0.7);
	scroll1.setPreferredSize(new Dimension(x1, YDIM));
	scroll2.setPreferredSize(new Dimension(x2, YDIM));
	
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
	panel.add(scroll1);
	panel.add(scroll2);
	getContentPane().add(panel);
    }

    private class HelpHTML extends JEditorPane {

	public HelpHTML(String file) {
	    
	    setEditable(false);

	    String s = System.getProperty("file.separator");
	    String path = "file:" + maui.home() + s + MAUI.DOC_DIR + s + MAUI.HELP_DIR + s + file;
	
	    try {
		URL url = new URL(path);
		setPage(url);
	    
	    } catch (Exception e) {
		// Ignore broken links
	    }
	}
    }

    private class Hyperactive implements HyperlinkListener {
	
	private HelpHTML html;

	public Hyperactive(HelpHTML h) {
	    html = h;
	}

	public void hyperlinkUpdate(HyperlinkEvent e) {
	    if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
		
		if (e instanceof HTMLFrameHyperlinkEvent) {
		    HTMLFrameHyperlinkEvent  evt = (HTMLFrameHyperlinkEvent)e;
		    HTMLDocument doc = (HTMLDocument) html.getDocument();
		    doc.processHTMLFrameHyperlinkEvent(evt);
		} else {
		    try {
			html.setPage(e.getURL());
		    } catch (Throwable t) {
			t.printStackTrace();
		    }
		}
	    }
	}
    }

}
