package org.uclic.maui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Properties;
import java.util.Vector;
import javax.swing.*;
import org.jdom.JDOMException;

/**
 * The main menu bar for the MAUI application.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class MAUIMenu extends JMenuBar {

    private MAUI maui;
 
    public final static String FILE_MENU  = "File";
    public final static String OPEN_FILE  = "Open";
    public final static String NEW_DESIGN = "New";
    public final static String SAVE_FILE  = "Save";
    public final static String EXIT_APP   = "Exit";

    public final static String HELP_MENU  = "Help";
    public final static String MAUI_HELP  = "Help";
    public final static String ABOUT_MAUI = "About MAUI";

    public final static String OPEN_ICN = "Open16.gif";
    public final static String NEW_ICN = "New16.gif";
    public final static String SAVE_ICN = "Save16.gif";
    public final static String HELP_ICN = "Help16.gif";
    public final static String ABOUT_ICN = "About16.gif";
 
    public MAUIMenu(MAUI m) {
        super();
        maui = m;		
        fileMenu();
        helpMenu();
    }	
	 
    // File Menu
    private void fileMenu() {
  		
        JMenu fileMenu = new JMenu(FILE_MENU);
  		
        FileOperator op = maui.operator();
  	
        menuItem(OPEN_FILE, OPEN_ICN, fileMenu, op, KeyEvent.VK_F, ActionEvent.CTRL_MASK);
	menuItem(NEW_DESIGN, NEW_ICN, fileMenu, maui);
        fileMenu.addSeparator();
        menuItem(SAVE_FILE, SAVE_ICN, fileMenu, op, KeyEvent.VK_S, ActionEvent.CTRL_MASK);
        fileMenu.addSeparator();
        fileMenu.addSeparator();
        menuItem(EXIT_APP, null, fileMenu, maui, KeyEvent.VK_Q, ActionEvent.CTRL_MASK);
        add(fileMenu);
    }
		
    // Help Menu
    private void helpMenu() {
        JMenu helpMenu = new JMenu(HELP_MENU);
        menuItem(MAUI_HELP, HELP_ICN, helpMenu, maui);
        menuItem(ABOUT_MAUI, ABOUT_ICN, helpMenu, maui);
        add(helpMenu);
    }
  
    private JMenuItem menuItem(String pName, String iconFile, JMenu pMenu, ActionListener pListener) {
                
	JMenuItem item = new JMenuItem(pName);
	if (iconFile != null) {
	    item.setIcon(new ImageIcon(maui.iconPath(iconFile)));
	}

        pMenu.add(item);
        item.setActionCommand(pName);
        item.addActionListener(pListener);
	
        return item;
    }
  
    private JMenuItem menuItem(String pName, String iconFile, JMenu pMenu, ActionListener pListener, int ke, int ae) {
        
	JMenuItem item = new JMenuItem(pName);
	if (iconFile != null) {
	    item.setIcon(new ImageIcon(maui.iconPath(iconFile)));
	}

        pMenu.add(item);
        item.setActionCommand(pName);
        item.addActionListener(pListener);
	item.setAccelerator(KeyStroke.getKeyStroke(ke, ae));
	
        return item;
    }
  

    private JMenuItem prefItem(String pName, JMenu pMenu) {
        String pref = maui.pref(pName, "");
        JMenuItem item = new JMenuItem(pref);
        File file = new File(pref);

        if (file.exists()) {
            pMenu.add(item);
            item.setActionCommand(pName);
            item.addActionListener(maui);
        }
        return	item;
    }

}
