package org.uclic.maui;

import java.awt.*;
import java.beans.*;
import java.net.URL;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;
import org.uclic.algebra.*;
import org.uclic.fui.*;
import org.uclic.maui.*;

/**
 * A generic tab for the MAUI application. 
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public abstract class MAUITab extends JPanel {
    
    protected MAUI      maui;
    protected String    title;
    protected ImageIcon icon;
    protected String    tip;

    protected boolean   changed;
    protected Vector    edits;

    public MAUITab(String t, MAUI m, String i) {
        this(t, m, i, null);
    }

    public MAUITab(String t, MAUI m, String i, String p) {
        super(); 
	maui = m;
	title = t;
	tip = p;
	String sep = System.getProperty("file.separator");
	if (i != null) {
	    icon = new ImageIcon(maui.iconPath(i));
	}	
	changed = false;
	edits = new Vector();
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
    }	

    public String title() {
	return title;
    }

    public ImageIcon icon() {
	return icon;
    }

    public String tip() {
	return tip;
    }

    public MAUI maui() {
	return maui;
    }

    public FUIDesign design() {
	return maui.design();
    }

    public SignFSM model() {
	return maui.design().model();
    }

    public EventAlgebra algebra() {
	return maui.design().algebra();
    }

    public Specification spec() {
	return maui.design().spec();
    }

    public void broadcastChange(int code) {
	maui.designChanged(code);
    }

    public void designChanged(int code) {
	changed = true;
	edits.add(new Integer(code));
    }   

    public void tabSelected() {

	maui.clearStatus();

	if (changed) {
	    rebuildTab();
	    changed = false;
	    edits.removeAllElements();
	} 
    } 

    public abstract void rebuildTab();

}
