package org.uclic.maui;

import java.awt.*;
import javax.swing.*;
import org.uclic.algebra.*;
import org.uclic.fui.*;
import org.uclic.maui.plugin.*;

/**
 * A tab for a MAUI plugin.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class PluginTab extends MAUITab {

    private MAUIPlugin plugin;
    private final static String TITLE = "Plugin";
    private final static String ICON  = "Import24.gif";
    private final static String TIP   = "";

    public PluginTab(MAUI m, MAUIPlugin p) {
	super(TITLE, m, ICON, TIP);
	plugin = p;
	plugin.init(new DataBridge(this));
	title = p.getTitle();
	tip = p.getTip();
	add(plugin.getComponent());
    }

    public void rebuildTab() {

	if (edits.contains(new Integer(MAUI.NEW_DESIGN))) {
	    plugin.mauiDesignLoaded();

	} else if (edits.contains(new Integer(MAUI.NEW_TITLE))) {
	    plugin.mauiTitleChanged();

	} else if (edits.contains(new Integer(MAUI.MODEL_EDIT))) {
	    plugin.mauiModelChanged();

	} else if (edits.contains(new Integer(MAUI.RENAME_EDIT))) {
	    plugin.mauiElementRenamed();

	} else if (edits.contains(new Integer(MAUI.SPEC_EDIT))) {
	    plugin.mauiSpecificationChanged();	    
	}

	removeAll();
	add(plugin.getComponent());
    }


    private class DataBridge implements MAUIBridge {

	private PluginTab tab;

	public DataBridge(PluginTab t) {
	    tab = t;
	}

	public FUIDesign getDesign() {
	    return design();
	}
	
	public boolean isSelected() {
	    return (maui.selected() == tab);
	}
	
	public void statusMessage(String message) {
	    if (message != null) {
		maui.setStatus(message);
	    }
	}

	public void fireMauiDesignChanged() {
	    broadcastChange(MAUI.NEW_DESIGN);
	}

	public void fireMauiTitleChanged() {
	    broadcastChange(MAUI.NEW_TITLE);
	}

	public void fireMauiModelChanged() {
	    broadcastChange(MAUI.MODEL_EDIT);
	}

	public void fireMauiElementRenamed() {
	    broadcastChange(MAUI.RENAME_EDIT);
	}

	public void fireMauiSpecificationChanged() {
	    broadcastChange(MAUI.SPEC_EDIT);
	}
    }
}
