package org.uclic.maui.analyse;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyVetoException;
import java.util.Date;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;
import org.jdom.Element;
import org.uclic.matrix.Matrix;
import org.uclic.matrix.WrongSize;
import org.uclic.algebra.*;
import org.uclic.fui.*;
import org.uclic.maui.*;

/**
 * An event algebra specification analysis tool.
 *
 * @author Jeremy Gow
 * @version 0.4
 * 
 * Abstract out column numbers.
 */
public class Analysis extends MAUITab implements ActionListener {

    private PropertyTable  table;
    //private JTextField     size;
    //private JTextField     partial;

    public  final static String NEW      = "Add";
    public  final static String DELETE   = "Delete";
    public  final static String EVALUATE = "Evaluate";
    public  final static String SUGGEST  = "Explore";
    private final static String TITLE    = "Specification";
    private final static String ICON     = "Properties24.gif";
    private final static String TIP      = "Specify the interface's behaviour";

    private final static String NEW_ICON   = "RowInsertAfter24.gif";
    private final static String DEL_ICON   = "RowDelete24.gif";
    private final static String EVAL_ICON  = "lightning24.gif";
    private final static String SUG_ICON   = "TipOfTheDay24.gif";

    private final static int DEFAULT_SIZE     = 3;
    private final static int DEFAULT_PARTIAL  = 90;
    private final static String SIZE_LABEL    = "Max. Size:";
    private final static String PARTIAL_LABEL = "Partiality:";

    public Analysis (MAUI a){
	super(TITLE, a, ICON, TIP);
	build();
    }

    private void build() {

	table = new PropertyTable(maui);

	JScrollPane scroller = new JScrollPane(table);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	
	JToolBar bar = new JToolBar(SwingConstants.VERTICAL);
	bar.setAlignmentX(Component.CENTER_ALIGNMENT);
	bar.setAlignmentY(Component.TOP_ALIGNMENT);
	bar.setFloatable(false);
	bar.add(buildButton(NEW, NEW_ICON));
	bar.add(Box.createRigidArea(new Dimension(0, 10)));
	bar.add(buildButton(DELETE, DEL_ICON));
	bar.add(Box.createRigidArea(new Dimension(0, 10)));
	bar.add(buildButton(EVALUATE, EVAL_ICON));
	bar.add(Box.createRigidArea(new Dimension(0, 50)));

	/* SUGGEST BUTTON
	bar.add(buildButton(SUGGEST, SUG_ICON));
	bar.add(Box.createRigidArea(new Dimension(0, 10)));

	size = new JTextField(2);
	bar.add(buildFieldPanel(size, SIZE_LABEL, DEFAULT_SIZE));

	partial = new JTextField(2);
	bar.add(buildFieldPanel(partial, PARTIAL_LABEL, DEFAULT_PARTIAL));
	*/

        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	add(bar);
	bar.add(Box.createRigidArea(new Dimension(10, 0)));
	add(scroller);
    }

    private JButton buildButton(String str, String file) {
	JButton button = new JButton(str);
	button.addActionListener(this);
	button.setActionCommand(str);
	sizeComponent(button, new Dimension(100,30));

        String sep = System.getProperty("file.separator");
	String path = maui.home() + sep + MAUI.ICONS + sep;
        button.setIcon(new ImageIcon(path + file));	

	return button;
    }

    private JPanel buildFieldPanel(JTextField f, String l, int d) {

	setField(f, d);
	JPanel panel = new JPanel();
	panel.add(new JLabel(l));
	panel.add(f);
	sizeComponent(panel, new Dimension(100,30));	
	return panel;
    }

    private void sizeComponent(JComponent c, Dimension d) {
	c.setMinimumSize(d);
	c.setMaximumSize(d);
	c.setPreferredSize(d);
	c.setAlignmentX(Component.CENTER_ALIGNMENT);
	c.setAlignmentY(Component.TOP_ALIGNMENT);
    }


    public void rebuildTab() {

	if (edits.contains(new Integer(MAUI.NEW_DESIGN))) {
	    removeAll();
	    build();
	} else {
	    table.designChanged(edits);
	}
    }

    public void actionPerformed(ActionEvent e) {

	String command = e.getActionCommand();

	if (command.equals(Analysis.NEW)) {
	    table.newProperty();
		    
	} else if (command.equals(Analysis.DELETE)) {
	    table.deleteProperties();
		    
	} else if (command.equals(Analysis.EVALUATE)) {
	    table.evaluate();

	} else if (command.equals(Analysis.SUGGEST)) {
	    //suggest();
	}
    }
    
    private void setField(JTextField f, int n) {
	Integer i = new Integer(n);
	f.setText(i.toString());
    }

    private int getField(JTextField f) {

	int n;

	try {
	    n = Integer.parseInt(f.getText());
	} catch (NumberFormatException e) {
	    n = -1;
	}
	
	return n;
    }

    /*
    private void suggest() {

	int maxSize = getField(size);
	if (maxSize < 1) {
	    maxSize = DEFAULT_SIZE;
	    setField(size, maxSize);
	}

	int minPart = getField(partial);
	if (!(0 < minPart || minPart < 100)) {
	    minPart = DEFAULT_PARTIAL;
	    setField(partial, minPart);
	}

	ExploreDialog dialog = new ExploreDialog(maui, maxSize, minPart);
    }
    */
}
