package org.uclic.maui.analyse;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.EventObject;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.uclic.maui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.property.*;

/**
 * Dialog for the automatic suggestion mechanism.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class ExploreDialog extends JDialog implements ActionListener {
    
    private MAUI maui;
    private int maxSize;
    private int minPart;
    private JTextArea log;

    private final static String TITLE  = "MAUI: Explore the Interface";
    private final static String ENOUGH = "Enough";
    private final static String CANCEL = "Cancel";

    public ExploreDialog(MAUI m, int s, int p) {
	super(m, TITLE);
	maui = m;
	maxSize = s;
	minPart = p;

	build();
	explore();
    }

    private void build() {

	log = new JTextArea();
	JScrollPane scroll = new JScrollPane(log);

	JButton enough = new JButton(ENOUGH);
	enough.addActionListener(this);
	enough.setActionCommand(ENOUGH);

	JButton cancel = new JButton(CANCEL);
	cancel.addActionListener(this);
	cancel.setActionCommand(CANCEL);

	JPanel buttons = new JPanel();
	buttons.add(enough);
	buttons.add(cancel);
	
	JPanel main = new JPanel();
	main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
	main.add(scroll);
	main.add(buttons);
	getContentPane().add(main);

	setSize(new Dimension(300,300));
	position();
	maui.setEnabled(false);
	setVisible(true);
    }

    private void position() {
	Point fp = maui.getLocation();
	Dimension fd = maui.getSize();
	Dimension d = getSize();

	double dx = (fd.getWidth() - d.getWidth())/2;
	double dy = (fd.getHeight() - d.getHeight())/2;

	int px = (int) (fp.getX() + dx);
	int py = (int) (fp.getY() + dy);
	
	setLocation(new Point(px, py));
    }


    public void actionPerformed(ActionEvent e) {
	
	maui.setEnabled(true);
	setVisible(false);
    }


    private void explore() {
		
	EventAlgebra alg = maui.design().algebra();

	GeneratorLog genLog = new GeneratorLog() {
		public void write(String str) {
		    log.append(str);
		}
	    };

	PropertyGenerator gen = new PropertyGenerator(alg, maxSize, minPart, genLog);
	gen.generate();
	Vector props = gen.properties();
	
    }   
    

	/*
	String msg;
	msg = "Max product length:    " + maxSize;
	msg = msg + "\nPartial theorem limit: " + minPart + "%";
	msg = msg + "\nSuggested theorems:  " + props.size();
	msg = msg + "\n\nAdd to specification?";
	
	int o = JOptionPane.showConfirmDialog(maui, msg, "Suggestion Results Summary",
					      JOptionPane.YES_NO_OPTION,
					      JOptionPane.QUESTION_MESSAGE);
	
	if (o == JOptionPane.YES_OPTION) {
	    
	    Date date = new Date();
	    System.out.println("Start adding to spec: " + date.toString());
	    
	    if (props != null) {
		//spec.addProperties(props);
	    }
	    
	    date = new Date();
	    System.out.println("End adding to spec: " + date.toString());
	}
	*/

    
}
