package org.uclic.maui.analyse;

import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.uclic.maui.*;
import org.uclic.fui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.property.*;

/**
 * Editor for event algebra properties.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class PartialData extends JDialog implements ActionListener, CellDialog {
    
    private MAUI          maui;
    private TableCellEditor editor;

    private JLabel line1;
    private JLabel line2;
    private Vector trueStates;
    private Vector falseStates;

    private final static String TITLE = "Partial Result";
    private final static String TIP   = "Click for details";
    private final static String ERROR_STATE = "Error: Unknown State";
    private final static String OK = "Ok";
    private final static String TRUE_TITLE = "True States";
    private final static String FALSE_TITLE = "False States";

    private final static int XDIM  = 400;
    private final static int YDIM  = 400;
    private final static int SPACE = 10;

    public PartialData(MAUI m) {
	super(m, TITLE);
	maui = m;

	JPanel blurb = new JPanel();
	blurb.setLayout(new BoxLayout(blurb, BoxLayout.Y_AXIS));
	blurb.setAlignmentX(Component.CENTER_ALIGNMENT);
	line1 = addLine(blurb);
	line2 = addLine(blurb);

	JPanel scrolls = new JPanel();
	scrolls.setLayout(new BoxLayout(scrolls, BoxLayout.X_AXIS));
	scrolls.setBorder(BorderFactory.createEmptyBorder(SPACE, SPACE, SPACE, SPACE));
	trueStates = new Vector();
	falseStates = new Vector();
	addScroll(TRUE_TITLE, trueStates, scrolls);
	scrolls.add(Box.createRigidArea(new Dimension(SPACE, 0)));
	addScroll(FALSE_TITLE, falseStates, scrolls);

	JButton ok = new JButton(OK);
	ok.addActionListener(this);
	ok.setAlignmentX(Component.CENTER_ALIGNMENT);

	Container pane = getContentPane();
	pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
	pane.add(Box.createRigidArea(new Dimension(0, SPACE)));
	pane.add(blurb);
	pane.add(scrolls);
	pane.add(ok);
	pane.add(Box.createRigidArea(new Dimension(0, SPACE)));
	setSize(new Dimension(XDIM, YDIM));
	position();
    }

    private JLabel addLine(JPanel panel) {
	JLabel line = new JLabel();
	line.setAlignmentX(Component.CENTER_ALIGNMENT);
	panel.add(line);
	return line;
    }

    private void addScroll(String title, Vector list, JPanel panel) {
	JScrollPane scroll = new JScrollPane(new JList(list));
	scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS); 
	scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	scroll.setColumnHeaderView(new JLabel(title));
	int x = (int) ((XDIM - (4 * SPACE)) / 2);
	int y = (int) ((YDIM - (4 * SPACE)) * 0.7); 
	scroll.setPreferredSize(new Dimension(x, y));
	panel.add(scroll);
    }



    public void registerEditor(TableCellEditor e) {
	editor = e;
    }

    public void actionPerformed(ActionEvent e) {
      	editor.cancelCellEditing();
    }

    public void configure(int i) {

	Specification spec = maui.design().spec();	
	PropertyTerm prop = spec.property(i);
	line1.setText(prop.termString());
	line2.setText("is true for " + spec.percent(i) + "% of states");

	try {
	    boolean states[] = prop.evaluate();	
	    for (int j = 0; j < states.length; j++) {
		if (states[j]) {
		    addState(trueStates, j);
		} else {
		    addState(falseStates, j);
		}
	    }

	} catch (EvaluationException e) {
	    line2.setText("Unable to compute the true states for this property");
	}	
    }


    private void addState(Vector list, int state) {
	try {
	    list.add(maui.design().model().state(state));

	} catch (FUIException e) {
	    list.add(ERROR_STATE);
	}
    }

    private void position() {
	Point fp = maui.getLocation();
	Dimension fd = maui.getSize();
	Dimension d = getSize();

	double dx = (fd.getWidth() - d.getWidth())/2;
	double dy = (fd.getHeight() - d.getHeight())/2;

	int px = (int) (fp.getX() + dx);
	int py = (int) (fp.getY() + dy);
	
	setLocation(new Point(px, py));
    }

    public Object getValue() {
	return null;
    }

    public String getTip() {
	return TIP;
    }
      

}
