package org.uclic.maui.analyse;

import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.uclic.maui.*;
import org.uclic.algebra.*;
import org.uclic.algebra.property.*;

/**
 * Editor for event algebra properties.
 *
 * @author Jeremy Gow
 * @version 0.4
 */
public class PropertyEditor extends JDialog implements ActionListener, CellEditorListener, CellDialog {
    
    private MAUI          maui;
    private PropertyTerm  property;
    private TreeModel     treemodel;
    private JTextField    text;
    private TableCellEditor editor;

    private final static String SET      = "Set";
    private final static String CANCEL   = "Cancel";
    private final static String TITLE    = "Property Editor";
    private final static String PROPERTY = "Property: ";
    private final static String SELECTED = "Selected: ";
    private final static String TREE     = "Term Tree";
    private final static String TIP      = "Click to edit";

    public PropertyEditor(MAUI m) {
	super(m, TITLE);
	maui = m;
    }

    public void registerEditor(TableCellEditor e) {
	editor = e;
    }

    public void configure(int i) {
	property = maui.design().spec().property(i);

	// Property panel
	JPanel propPanel = new JPanel();
	propPanel.setLayout(new BoxLayout(propPanel, BoxLayout.X_AXIS));
	text = new JTextField(30);
	text.setEditable(false);
	text.setBackground(Color.WHITE);
	propPanel.add(new JLabel(PROPERTY));
	propPanel.add(text);

	// Term tree
	TermTree tree = new TermTree(property, this);
	treemodel = tree.getModel();
	JScrollPane scroller = new JScrollPane(tree);
	scroller.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), TREE));
	
	// Buttons
	JPanel setPanel = new JPanel();
	setPanel.add(buildButton(SET, this));
	setPanel.add(buildButton(CANCEL, this));	

	// Main panel
	JPanel panel = new JPanel();
	panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	panel.add(propPanel);
	panel.add(scroller);
	panel.add(setPanel);
	getContentPane().add(panel);

	// Configure
	showProperty();
	setSize(new Dimension(300,300));
	position();
    }

    public JButton buildButton(String name, ActionListener a) {
	JButton btn = new JButton(name);
	btn.addActionListener(a);
	btn.setActionCommand(name);
	return btn;
    }

    private void position() {
	Point fp = maui.getLocation();
	Dimension fd = maui.getSize();
	Dimension d = getSize();

	double dx = (fd.getWidth() - d.getWidth())/2;
	double dy = (fd.getHeight() - d.getHeight())/2;

	int px = (int) (fp.getX() + dx);
	int py = (int) (fp.getY() + dy);
	
	setLocation(new Point(px, py));
    }

    public Object getValue() {
	return property;
    }

    public String getTip() {
	return TIP;
    }

    public void showProperty() {
	if (property != null) {
	    text.setText(property.termString());
	} else {
	    text.setText("NULL PROPERTY");
	}
    }

    public void actionPerformed(ActionEvent e) {
	    
	String cmd = e.getActionCommand();

	if (cmd.equals(SET)) {
	    editor.stopCellEditing();
	} else if (cmd.equals(CANCEL)) {
	    editor.cancelCellEditing();
	}		
    }

    public void editingCanceled(ChangeEvent e) {}

    public void	editingStopped(ChangeEvent e) {
	property = (PropertyTerm) treemodel.getRoot();	
	showProperty();
    }
	
	
}
