package org.uclic.maui.edit;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.uclic.fui.*;
import org.uclic.maui.*;
import org.uclic.util.ArrayUtils;

/**
 * @author Jeremy Gow
 * @version 0.4
 *
 */
public abstract class EditorPanel extends JPanel {

    protected JButton title;
    protected boolean selected;
    protected Editor  editor;
    protected Color   col;
    protected Color   scol;

    public final static String SELECT = "Select";
    
    public EditorPanel(String name, JPanel parent, Editor ed) {
	super();
	editor = ed;
	
	// Add title button
	title = new JButton(name);
	title.addActionListener(ed);
	title.setActionCommand(name);
	title.setAlignmentX(Component.CENTER_ALIGNMENT);
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
       	add(Box.createRigidArea(new Dimension(0, 5)));
	add(title);
	
	col = title.getBackground();
	scol = col.darker();
	setBackground(scol);
 
	// Add to parent panel
	parent.add(this);

	// Panel initially selected
	selected = true;
    }

    public boolean isSelected() {
	return selected;
    }

    public SignFSM model() {
	return editor.model();
    }

    public FUIDesign design() {
	return editor.design();
    }

    public SuperPanel superPanel() {
	return editor.superPanel;
    }

    public StatePanel statePanel() {
	return editor.statePanel;
    }

    public EventPanel eventPanel() {
	return editor.eventPanel;
    }

    public RangePanel rangePanel() {
	return editor.rangePanel;
    }

    public SignsPanel signsPanel() {
	return editor.signsPanel;
    }

    
    /**
     * Select panel.
     *
     */
    protected void select() {

	if (!selected) {
	    // Set this panel as selected
	    selected = true;
	    
	    // Panel header indicates selection
	    title.setFont(title.getFont().deriveFont(Font.BOLD));
	    setBackground(scol);
	}
    }
    
    /**
     * Deselect panel.
     * 
     */
    protected void deselect() {
	
	if (selected) {
	    // Set as unselected
	    selected = false;		
	    
	    // Remove selection indicators from panel header 
	    title.setFont(title.getFont().deriveFont(Font.PLAIN));
	    setBackground(col);
	}
    }
        
    public void setTitleEnabled(boolean e) {
	if (isEnabled() != e) {
	    setEnabled(e);
	}
	title.setEnabled(e);
    }

}
