package org.uclic.maui.edit;

import javax.swing.JPanel;
import org.uclic.fui.FUIException;
import org.uclic.fui.FUI_XML;
import org.uclic.util.ArrayUtils;

/**
 * @author Jeremy Gow
 * @version 0.4
 */
public class EventPanel extends ListPanel {

    private StateTrans  stateReln;
    private SuperTrans  superReln;

    public EventPanel(JPanel panel, Editor ed) {
	super(Editor.EVENTS, panel, ed);
	stateReln = new StateTrans();
	superReln = new SuperTrans();
    }

    public void addItem(int i) throws FUIException {
	model().addEvent(FUI_XML.EVENT_ELEM + i);
    }

    public void deleteItem(int i) {
	design().deleteEvent(i);
    }

    public void renameItem(int i, String name) throws FUIException {
	model().renameEvent(i, name);
    }

    public String[] items() {
	return model().events();
    }

    public String getItemType() {
	return Editor.EVENT_LABEL;
    }

    public void highlightRelated() {
	    
	if (editor.isTransSelected()) {
	    highlightStateDomain();
	    highlightSuperDomain();

	} else if (statePanel().isSelected()) {
	    statePanel().highlightRange();

	} else if (superPanel().isSelected()) {
	    superPanel().highlightRange();

	}
    }

    public void highlightStateDomain() {
	highlightList(rangePanel().list, statePanel().list, stateReln, true);
    }

    public void highlightSuperDomain() {
	highlightList(rangePanel().list, superPanel().list, superReln, true);
    }

    public void editModel() {
	if (statePanel().isSelected()) {
	    editModel(statePanel(), rangePanel(), stateReln);

	} else if (superPanel().isSelected()) {
	    editModel(superPanel(), rangePanel(), superReln);
	}
    }

    private class StateTrans implements TertiaryRelation {

	public int[] map(int event, int range, boolean ignore) {
	    return model().domainIndices(event, range);
	}

	public void toggle(int event, int state, int range) {
	    model().toggleTransition(state, event, range);
	}
    }

    private class SuperTrans implements TertiaryRelation {

	public int[] map(int event, int range, boolean ignore) {
	    return model().superDomainIndices(event, range);
	}
    	
	public void toggle(int event, int superstate, int range) {
	    model().toggleSuperTransition(superstate, event, range);
	}
    }
}
