package org.uclic.maui.edit;

import java.awt.*;
import javax.swing.*;

/**
 * List which allows two modes:
 * <ul>
 *   <li>
 *   Multi mode.  Mulitple items can be selected.
 *    Only selected items are highlighted.
 *   </li>
 *   <li>
 *   Single mode.  Only a single item can be selected.
 *   A separate list of highlighted items is
 *   maintained.  Selection toggles highlighting.
 *   </li>
 * </ul>
 *
 */
public class HighlightList extends JList {
   
    private boolean   multiMode;
    private boolean[] pred;
    private boolean[] reserve;
    private Color     color;
    private ListPanel panel;
    //private int       editing;

    public HighlightList(ListPanel p) {
	super();
	panel = p;
	//editing = -1;
	setCellRenderer(new EditorCellRenderer());
	color = getSelectionBackground();
	clearHighlights();
	reserve = null;
	setMultiMode();
    }
    
    /** 
     * This mode allows multiple selection, in red.
     *
     */
    public void setMultiMode() {
	setSelectionBackground(Color.RED);
	setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
	
	// Turn highlighted to selected
	if (pred != null) {
	    for (int i = 0; i < pred.length; i++) {
		if (pred[i]) {
		    setSelectedIndex(i);
		}
	    }
	} else {
	    clearSelection();
	}
	
	multiMode = true;
    }

    /**
     * This mode allows single selection and toggled highlighting,
     * in the default selection colour.
     *
     */
    public void setSingleMode() {
	setSelectionBackground(color);
	setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	multiMode = false;
    }
    
    public boolean isIndex(int i) {
	return -1 < i && i < pred.length;
    }

    
    public boolean isHighlighted(int i) {
	
	if (isIndex(i)) {
	    return pred[i];
	} else {
	    return false;
	}
    }

    public void highlight(boolean[] predicate) {
	
	if (predicate.length == getModel().getSize()) {
	    pred = predicate;
	    repaint();
	}
     }
    
    public void highlight(int[] indices) {
	
	pred = new boolean[getModel().getSize()];
	if (indices != null) {
	    for (int i = 0; i < indices.length; i++) {
		if (isIndex(indices[i])) {	       
		    pred[indices[i]] = true;
		}
	    }
	}
	repaint();
    }
    
    public void clearHighlights() {
	pred = new boolean[getModel().getSize()];
	repaint();
    }

    public void clearAll() {
	clearSelection();
	clearHighlights();
    }
    
    public void reserveHighlights() {
	if (pred == null) {
	    reserve = null;
	} else {
	    reserve = new boolean[pred.length];
	    System.arraycopy(pred, 0, reserve, 0, pred.length);
	}
    }

    public void restoreHighlights() {
	pred = reserve;
    }

    /*
    public void startEditing(int index) {
	editing = index;
	repaint();
    }

    public void stopEditing () {
	if (editing != -1) {
	    editing = -1;
	    repaint();
	}
    }
    */
    


    /**
     * INNER CLASS: EditorCellRenderer
     *
     * Enforces highlighting of items when in single mode.
     *
     */
    private class EditorCellRenderer extends DefaultListCellRenderer implements ListCellRenderer {
		
	public Component getListCellRendererComponent(JList l, Object o, int i, boolean s, boolean h) {
	    Component c = super.getListCellRendererComponent(l, o, i, s, h);
	    		 
	    /*
	    if (i == editing) {
		JTextField t = new JTextField("RENAME");
		t.setEditable(true);
		//t.grabFocus();
		//t.requestFocusInWindow();
		return t;
		    
	    } else 
	    */

	    if (!multiMode) {
		if (isHighlighted(i)) {
		    c.setBackground(color);
		} else {
		    c.setBackground(Color.WHITE);
		}
	    } else {
		if (!isSelectedIndex(i)) {
		    //c.setBackground(color);
		}
	    }

	    return c;
	}
    }

}
