package org.uclic.maui.edit;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.uclic.fui.*;
import org.uclic.maui.*;
import org.uclic.util.ArrayUtils;

/**
 * @author Jeremy Gow
 * @version 0.4
 *
 */
public abstract class ListPanel extends EditorPanel implements ActionListener, ListSelectionListener {

    public    HighlightList list;
    protected JButton plus;
    protected JButton minus;
    protected JPanel  editControls;

    private final static String ADD = "Add";
    private final static String DEL = "Remove";
    
    public ListPanel(String name, JPanel panel, Editor ed) {
	super(name, panel, ed);

	plus = new JButton(ADD);
	minus = new JButton(DEL);
	
	plus.addActionListener(this);
	plus.setActionCommand(ADD);
	
	minus.addActionListener(this);
	minus.setActionCommand(DEL);
	
	editControls = new JPanel();
	editControls.add(plus);
	editControls.add(minus);
	editControls.setAlignmentX(Component.CENTER_ALIGNMENT);
	editControls.setBackground(scol);
	add(editControls);
	
	list = new HighlightList(this);
	list.addListSelectionListener(this);
	//list.addMouseListener(new DoubleClick());

	JScrollPane scroll = new JScrollPane();
	scroll.setViewportView(list);
	scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS); 
	scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	setBorder(BorderFactory.createEtchedBorder());
	add(scroll);
	
	int minX = (int) (MAUI.XDIM * 0.19);
	int minY = (int) (MAUI.YDIM * 0.7);	
	setPreferredSize(new Dimension(minX, minY));

	int listMinX = (int) (MAUI.XDIM * 0.1);
	int listMinY = (int) (MAUI.YDIM * 0.6);	
	scroll.setPreferredSize(new Dimension(listMinX, listMinY));
    }

    /*
    private class DoubleClick extends MouseAdapter {
		
	public void mouseClicked(MouseEvent e) {
	    if (e.getClickCount() == 2) {
		int index = list.locationToIndex(e.getPoint());
		list.startEditing(index);
	    }
	}
    }
    */

    protected abstract String[] items();
    protected abstract void addItem(int i) throws FUIException;
    protected abstract void deleteItem(int i);
    protected abstract void renameItem(int i, String name) throws FUIException;
    protected abstract void highlightRelated();
    protected abstract void editModel();
    
    protected void select() {

	if (!selected) {
	    editor.setListening(false);
	    list.setMultiMode();
	    editor.setListening(true);
	    highlightRelated();
	    editor.setLastList(this);

	    Font f = plus.getFont().deriveFont(Font.BOLD);
	    plus.setFont(f);
	    minus.setFont(f);
	    minus.setEnabled(true);
	    editControls.setBackground(scol);
	}
	super.select();
    }
    
    protected void deselect() {
	
	if (selected) {
	    list.setSingleMode();

	    Font f = plus.getFont().deriveFont(Font.PLAIN);
	    plus.setFont(f);
	    minus.setFont(f);
	    minus.setEnabled(false);
	    editControls.setBackground(col);
	}
	super.deselect();
    }

    public void setEnabled(boolean e) {
	super.setEnabled(e);
	plus.setEnabled(e);
	minus.setEnabled(e);
	list.setEnabled(e);
    }
    
    protected void populateList() {
	list.setListData(items());
    }

    public String getSelectedItem() {
	return (String) list.getSelectedValue();
    }

    public abstract String getItemType();

    public void renameItem(String name) {
	int i = list.getSelectedIndex();
	if (i != -1) {
	    try {
		renameItem(i, name);
		editor.maui().designChanged(MAUI.RENAME_EDIT);
		repaint();

	    } catch (FUIException e) {
		String old = getItemType() + " " + getSelectedItem();
		editor.maui().setStatus("Cannot rename " + old + " to " + name);
	    }
	}
    }
    
    
    /**
     * Respond to user action from editor panel.
     *
     */
    public void actionPerformed(ActionEvent e) {
		
	if (editor.isListening()) {
	    String cmd = e.getActionCommand();
	    
	    if (cmd.equals(ADD)) {
		// ACTION: ADD ITEM
		
		// Try to add x1, x2, ... etc until novel name is found. 
		boolean todo = true;
		int n = 1;
		while(todo) { 
		    try {
			// Is the name novel...?
			addItem(n);			
			// Yes
			todo = false;
			populateList();
			editor.maui().designChanged(MAUI.MODEL_EDIT);
			
		    } catch (FUIException fe) {
			// No - so try next name.
			n++;
		    }
		}
		
	    } else if (cmd.equals(DEL)) {
		// ACTION: DELETE ITEM
		
		// Remove all selected items.
		int[] inds = list.getSelectedIndices();
		// Can't remove ZERO or ALL items
		if (inds.length != 0) {

		    if (inds.length < items().length) {
			for (int i = (inds.length - 1); i > -1; i--) {
			    deleteItem(inds[i]);
			}
			populateList();		    
			editor.maui().designChanged(MAUI.MODEL_EDIT);
			
		    } else {
			editor.maui().setStatus("Can't removes all items from a list");
		    }
		} else {
		    editor.maui().setStatus("No items selected");
		}
	    }
	}
    }
    
    private String uniqueName(String prefix, String[] array) {
	
	int n = 0;
	boolean unique = false;
	
	while (!unique) {
	    n++;
	    unique = true;
	    for (int i = 0; i < array.length; i++) {
		if (array[i].equals(prefix + n)) {
		    unique = false;
		}	
	    }
	}
	return prefix + n;
    }
    
    public void valueChanged(ListSelectionEvent e) {
	
	//list.stopEditing();

	if (editor.isListening() && !e.getValueIsAdjusting()) {
	    
	    editor.maui().clearStatus();
	    
	    if (selected) {
		highlightRelated();
		editor.setLastList(this);
	    } else {
		list.reserveHighlights();
		editor.design().reserveModel();
		editModel();
		editor.protectLocks(list);
	    }
	}
    }

    protected interface BinaryRelation {
	
	public boolean[] map(int i) throws FUIException;
	public void toggle(int i, int j);
    }

    protected interface TertiaryRelation {
	
	public int[] map(int i, int j, boolean b);
	public void toggle(int i, int j, int k);
    }

    /** 
     * Highlight elements of list. 
     */
    protected void highlightList(HighlightList l, BinaryRelation p) {
	
	    int[] sel = list.getSelectedIndices();
	    
	    if (sel.length == 0) {
		l.clearHighlights();

	    } else {

		try {
		    boolean[] pred = p.map(sel[0]);
		    
		    for (int i = 1; i < sel.length; i++) {
			boolean[] p2 = p.map(sel[i]);
			pred = ArrayUtils.andArray(pred, p2);
		    }

		    l.highlight(pred);
		} catch (FUIException e) {
		    // Selected index does not exist - don't cause a fuss.
		}   
	    }
    }

    /* Highlight elements of list.
     */
    public void highlightList(HighlightList list2, HighlightList list3, TertiaryRelation reln, boolean one) {
	
	int[] sel = list.getSelectedIndices();
	
	if (sel.length == 0) {
	    list3.clearHighlights();
	    
	} else {
	    int[] sel2 = list2.getSelectedIndices();
	    
	    if (sel2.length == 0) {
		list3.clearHighlights();
		
	    } else {
		int[] sel3 = reln.map(sel[0], sel2[0], one);
		
		for (int i = 1; i < sel2.length; i++) {
		    sel3 = ArrayUtils.intersection(sel3, reln.map(sel[0], sel2[i], one));
		}

		for (int i = 1; i < sel.length; i++) {
		    for (int j = 1; j < sel2.length; j++) {
			sel3 = ArrayUtils.intersection(sel3, reln.map(sel[i], sel2[j], one));
		    }
		}
		list3.highlight(sel3);
	    }		
	}	    
    }

    /* Toggle part of a binary relation, with this list as one argument.
     */
    public void editModel(ListPanel list2, BinaryRelation reln) {

	int   sel  = list.getSelectedIndex();	
	int[] sel2 = list2.list.getSelectedIndices();
	
	for (int i = 0; i < sel2.length; i++) {
	    reln.toggle(sel, sel2[i]);
	}

	editor.setListening(false);
	list.clearSelection();
	editor.setListening(true);
	list2.highlightRelated();
    }

    /* Toggle part of a tertiary relation, with this list as one argument.
     */
    public void editModel(ListPanel list2, ListPanel list3, TertiaryRelation reln) {

	int   sel  = list.getSelectedIndex();	
	int[] sel2 = list2.list.getSelectedIndices();
	int[] sel3 = list3.list.getSelectedIndices();
	
	for (int i = 0; i < sel2.length; i++) {
	    for (int j = 0; j < sel3.length; j++) {
		reln.toggle(sel, sel2[i], sel3[j]);
	    }
	}
	
	editor.setListening(false);
	list.clearSelection();
	editor.setListening(true);
	list2.highlightRelated();
    }

}
