package org.uclic.maui.edit;

import javax.swing.JPanel;
import org.uclic.fui.FUIException;
import org.uclic.fui.FUI_XML;
import org.uclic.util.ArrayUtils;

/**
 * @author Jeremy Gow
 * @version 0.4
 */
public class StatePanel extends ListPanel {

    private Superstates superReln;
    private StateTrans  transReln;
    private StateSigns  signReln;

    public StatePanel(JPanel p, Editor ed) {
	super(Editor.STATES, p, ed);
	superReln = new Superstates();
	transReln = new StateTrans();
	signReln = new StateSigns();
    }
    
    public void addItem(int i) throws FUIException {
	model().addState(FUI_XML.STATE_ELEM + i);
    }
    
    public void deleteItem(int i) {
	design().deleteState(i);
    }
    
    public void renameItem(int i, String name) throws FUIException {
	model().renameState(i, name);
    }
    
    public String[] items() {
	return model().states();
    }

    public String getItemType() {
	return Editor.STATE_LABEL;
    }
    
    protected void populateList() {
	list.setListData(items());
	rangePanel().list.setListData(items());
    }

    /**
     * Highlight related items.
     *
     */
    protected void highlightRelated() {

	highlightSuperstates();
	highlightSigns();

	if (eventPanel().isSelected()) {
	    highlightRange();
	} else {
	    highlightEvents();
	}
    }

    public void highlightSuperstates() {
	highlightList(superPanel().list, superReln);
    }

    public void highlightSigns() {
	highlightList(signsPanel().list, signReln);
    }

    public void highlightRange() {
	highlightList(eventPanel().list, rangePanel().list, transReln, true);
    }

    public void highlightEvents() {
	highlightList(rangePanel().list, eventPanel().list, transReln, false);
    }

    public void editModel() {
	if (superPanel().isSelected()) {
	    editModel(superPanel(), superReln);

	} else if (editor.isTransSelected()) {
	    editModel(eventPanel(), rangePanel(), transReln);

	} else if (signsPanel().isSelected()) {
	    editModel(signsPanel(), signReln);
	}
    }

    private class Superstates implements BinaryRelation {
	    
	public boolean[] map(int state) {
	    return model().superstates(state);
	}

	public void toggle(int state, int superstate) {
	    model().toggleIncluded(state, superstate);
	}
    }

    private class StateTrans implements TertiaryRelation {

	public int[] map(int state, int i, boolean range) {
	    if (range) {
		return model().rangeIndices(state, i);
	    } else {
		return model().mapsIndices(state, i);
	    }
	}

	public void toggle(int state, int event, int range) {
	    model().toggleTransition(state, event, range);
	} 
   }
	
    private class StateSigns implements BinaryRelation {
	    
	public boolean[] map(int state) {
	    return model().stateSigns(state);
	}	    

	public void toggle(int state, int sign) {
	    model().toggleStateSign(state, sign);
	}
    }
    
}
