package org.uclic.maui.edit;

import javax.swing.JPanel;
import org.uclic.fui.FUIException;
import org.uclic.fui.FUI_XML;
import org.uclic.util.ArrayUtils;

/**
 * @author Jeremy Gow
 * @version 0.4
 */
public class SuperPanel extends ListPanel {
    
    private SuperElements elemReln;
    private SuperSigns    signReln;
    private SuperTrans    transReln;

    public SuperPanel(JPanel p, Editor ed) {
	super(Editor.SUPERS, p, ed);
	elemReln = new SuperElements();
	signReln = new SuperSigns();
	transReln = new SuperTrans();
    }
    
    public void addItem(int i) throws FUIException {
	model().addSuperstate(FUI_XML.SUPER_REF_ELEM + i);
    }

    public void deleteItem(int i) {
	design().deleteSuperstate(i);
    }

    public void renameItem(int i, String name) throws FUIException {
	model().renameSuperstate(i, name);
    }

    public String[] items() {
	return model().superstates();
    }

    public String getItemType() {
	return Editor.SUPER_LABEL;
    }

    protected void highlightRelated() {

	// Highlight states and signs
	highlightList(statePanel().list, elemReln);
	highlightList(signsPanel().list, signReln);

	// Highlight related range states OR related events.
	if (eventPanel().isSelected()) {
	    highlightRange();
	} else {
	    highlightEvents();
	}
    }

    public void highlightRange() {
	highlightList(eventPanel().list, rangePanel().list, transReln, true);
    }

    public void highlightEvents() {
	highlightList(rangePanel().list, eventPanel().list, transReln, false);
    }

    public void editModel() {
	    
	int superstate = list.getSelectedIndex();
	    
	if (statePanel().isSelected()) {
	    editModel(statePanel(), elemReln);

	} else if (editor.isTransSelected()) {
	    editModel(eventPanel(), rangePanel(), transReln);

	} else if (signsPanel().isSelected()) {
	    editModel(signsPanel(), signReln);
	}
    }

    private class SuperElements implements BinaryRelation {
	    
	public boolean[] map(int superstate) throws FUIException {
	    return model().elements(superstate);
	}

	public void toggle(int superstate, int state) {
	    model().toggleIncluded(state, superstate);
	}
    }

    private class SuperSigns implements BinaryRelation {
	    
	public boolean[] map(int superstate) throws FUIException {
	    return model().superstateSigns(superstate);
	}

	public void toggle(int superstate, int sign) {
	    model().toggleSuperstateSign(superstate, sign);
	}
    }

    private class SuperTrans implements TertiaryRelation {

	public int[] map(int superstate, int i, boolean range) {
	    if (range) {
		return model().superRangeIndices(superstate, i);
	    } else {
		return model().superMapsIndices(superstate, i);
	    }
	}

	public void toggle(int superstate, int event, int range) {
	    model().toggleSuperTransition(superstate, event, range);	    
	}
    }
}
