package org.uclic.maui.plugin;

import java.awt.Component;

/**
 * <p>
 * An abstract adapter class for MAUI plugins.  The methods in this
 * class are empty.
 * </p>
 * <p>
 * Extend this class to create a MAUIPlugin.  The abstract methods
 * getTitle and getComponent must be defined by the subclass, but
 * others may be left undefined.
 * </p>
 */
public abstract class MAUIPluginAdapter implements MAUIPlugin {

    private MAUIBridge bridge;

    /*
     * Invoked when the MAUI application is started.
     */
    public void init(MAUIBridge b) {
	bridge = b;
    }

    protected MAUIBridge getBridge() {
	return bridge;
    }

    /*
     * Invoked when the MAUI application quits.
     */
    public void quit() {};
    
    /*
     * This MUST be implemented by the subclass.  Should return a 
     */
    public abstract String getTitle();

    /*
     * Invoked when the MAUI application is started.
     */

    public String getTip() {
	return "";
    };
    
    /*
     * This MUST be implemented by the subclass.
     */
    public abstract Component getComponent();

    public void mauiDesignLoaded() {};
    public void mauiTitleChanged() {};
    public void mauiModelChanged() {};
    public void mauiElementRenamed() {};
    public void mauiSpecificationChanged() {};

}
