package org.uclic.maui.skins;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.io.*;
import java.util.Observer;
import java.util.Observable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import org.apache.batik.swing.*;
import org.apache.batik.swing.gvt.*;
import org.apache.batik.swing.svg.*;
import org.w3c.dom.svg.*;
import org.uclic.maui.*;

/**
 * View the design's SVG skins.
 *
 * @author Jeremy Gow
 * @version 0.4
 *
 */
public class Skins extends MAUITab implements ListSelectionListener {

    private JSVGCanvas canvas;
    private JComboBox  skins;
    private boolean    notifyStatus;

    private final static String TITLE = "Skins";
    private final static String SELECT = "Skin: ";
    private final static String ICON  = "Movie24.gif";
    private final static String TIP   = "Configure skins";

    public Skins(MAUI a){
	super(TITLE, a, ICON, TIP);
	notifyStatus = false;
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	build();	
	
    }

    public void build() {

	// Skin selector
	skins = new JComboBox(maui.design().skins());
	skins.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {skinChosen();}
	    });
	skins.setBackground(Color.WHITE);

	JPanel skinPanel = new JPanel();
	skinPanel.add(new JLabel(SELECT));
	skinPanel.add(skins);


	// SVG Canvas
	canvas = new JSVGCanvas();
	canvas.setRecenterOnResize(true);
	canvas.setPreferredSize(new Dimension(300,300));
	canvas.setBorder(BorderFactory.createLineBorder(Color.black));
	canvas.addGVTTreeBuilderListener(new GVTTreeBuilderAdapter() {
		public void gvtBuildCompleted(GVTTreeBuilderEvent e) {setSize();}
	    });
	canvas.addSVGDocumentLoaderListener(new SVGDocumentLoaderAdapter() {
		public void documentLoadingCompleted(SVGDocumentLoaderEvent e) {setDOM();}
	    });
	canvas.addGVTTreeRendererListener(new GVTTreeRendererAdapter() {
		public void gvtRenderingCompleted(GVTTreeRendererEvent e) {
		    //System.out.println("Rendered");
		    if (notifyStatus) {
			maui.setStatus("");
			notifyStatus = false;
		    }
		}
	    });
	JPanel svgPanel = new JPanel();
	svgPanel.add(canvas);
	
	add(skinPanel);	
	add(svgPanel);
	
	if (skins.getItemCount() > 0) {
	    skins.setSelectedIndex(0);
	    setSkin();
	}

    /*
      JScrollPane scroll = new JScrollPane(skins);
      scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS); 
      scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      //setBorder(BorderFactory.createEtchedBorder());
      */
    }

    public void rebuildTab() {
	build();
	
    }

    public void skinChosen() {
	maui.setStatus("Rendering SVG...");
	notifyStatus = true; 
	setSkin();
    }

    public void setSkin() {
	//System.out.println("Set skin");
       
	Object o = skins.getSelectedItem();
	if (o != null) {
	    try {
		String skin = (String) o;
		File f = new File(skin);
		canvas.setURI(f.toURL().toString());
	    } catch (IOException ex) {
		ex.printStackTrace();
	    }	
	}
    }

    public void setSize() {
	//System.out.println("Set size");
	Dimension2D size = canvas.getSVGDocumentSize();
	int w = (int) size.getWidth();
	int h = (int) size.getHeight();
	setPreferredSize(new Dimension(w, h));
	//repaint();
    }

    public void setDOM() {
	//System.out.println("Set DOM");

	SVGDocument svgDoc = canvas.getSVGDocument();
    }

    public void valueChanged(ListSelectionEvent e) {
	

    }

}
