package org.uclic.util;

/**
 * Various array utilities.
 *
 * @author Jeremy Gow
 * @version 0.3
 *
 */
public final class ArrayUtils {

    /*
     * Is element in array?
     */
    public static boolean inArray(int e, int[] array) {

	if (array != null) {
	    for (int i = 0; i < array.length; i++) {
		if (array[i] == e) {
		    return true;
		}
	    }
	}
	return false;
    }

    /*
     * Find the index of String array element.
     */
    public static int findIndex(String s, String[] array) {
	int index = -1;

	for (int i = 0; i < array.length; i++) {
	    if (array[i].equals(s)) {
		index = i;
		break;
	    }
	}
	return index;
    }

    /*
     * Add an element to a String array.
     */
    public static String[] addToArray(String name, String[] array) {

	String[] newArray = new String[array.length + 1];
	System.arraycopy(array, 0, newArray, 0, array.length);
	newArray[array.length] = name;
	
	return newArray;
    }

    public static int[] addToArray(int i, int[] array) {

	int[] newArray = new int[array.length + 1];
	System.arraycopy(array, 0, newArray, 0, array.length);
	newArray[array.length] = i;
	
	return newArray;
    }

    /*
     * Add a row to an array table of int arrays
     */
    public static int[][][] addRowToArrayTable(int[][][] table) {

	int[][][] newTable = new int[table.length + 1][][];
	System.arraycopy(table, 0, newTable, 0, table.length);
	newTable[table.length] = new int[table[0].length][];

	return newTable;
    }

    /*
     * Add a row to an array table of booleans
     */
    public static boolean[][] addRowToArrayTable(boolean[][] table) {
	boolean[][] newTable = new boolean[table.length + 1][table[0].length];
	System.arraycopy(table, 0, newTable, 0, table.length);
	return newTable;
    }


    public static int[][][] addColumnToArrayTable(int[][][] table) {

	int n = table[0].length;

	for (int i = 0; i < table.length; i++) {
	    int[][] row = new int[n + 1][];
	    System.arraycopy(table[i], 0, row, 0, n);
	    table[i] = row;
	}

	return table;
    }

    public static boolean[][] addColumnToArrayTable(boolean[][] table) {

	int n = table[0].length;
	boolean[][] newTable = new boolean[table.length][n + 1];
	for (int i = 0; i < table.length; i++) {
	    System.arraycopy(table[i], 0, newTable[i], 0, n);
	}
	return newTable;
    }

    public static String[] deleteFromArray(int i, String[] array) {

	String[] newArray = new String[array.length - 1];
	System.arraycopy(array, 0, newArray, 0, i);
	if (i < newArray.length) {
	    System.arraycopy(array, i + 1, newArray, i, newArray.length - i);
	}
	return newArray;
    }

    public static int[] deleteFromArray(int i, int[] array) {
	
	int[] newArray = null;

	if (array.length > 1) {
	    newArray = new int[array.length - 1];
	    System.arraycopy(array, 0, newArray, 0, i);
	    if (i < newArray.length) {
		System.arraycopy(array, i + 1, newArray, i, newArray.length - i);
	    }
	    	    
	} 

	return newArray;
    }

    public static int[][][] deleteRowFromArrayTable(int i, int[][][] table) {

	int[][][] newTable = new int[table.length - 1][][];	
	System.arraycopy(table, 0, newTable, 0, i);
	if (i < newTable.length) {
	    System.arraycopy(table, i + 1, newTable, i, newTable.length - i);
	}

	return newTable;
    }

    public static boolean[][] deleteRowFromArrayTable(int row, boolean[][] table) {

	boolean[][] newTable = new boolean[table.length - 1][table[0].length];
	System.arraycopy(table, 0, newTable, 0, row);
	if (row < newTable.length) {
	    System.arraycopy(table, row + 1, newTable, row, newTable.length - row);
	}
	
	return newTable;
    }    

    public static int[][][] deleteColumnFromArrayTable(int i, int[][][] table) {

	int[][][] newTable = new int[table.length][table[0].length - 1][];

	for (int j = 0; j < newTable.length; j++) {
	    System.arraycopy(table[j], 0, newTable[j], 0, i);
	    if (i < newTable[j].length) { 
		System.arraycopy(table[j], i + 1, newTable[j], i, newTable[j].length - i);
	    }
	}

	return newTable;
    }

    public static boolean[][] deleteColumnFromArrayTable(int i, boolean[][] table) {
    
	boolean[][] newTable = new boolean[table.length][table[0].length - 1];

	for (int j = 0; j < table.length; j++) {
	    System.arraycopy(table[j], 0, newTable[j], 0, i);
	    if (i < newTable[j].length) {
		System.arraycopy(table[j], i + 1, newTable[j], i, newTable[j].length - i);
	    }
	}
	return newTable;
    }

    public static int[][][] deleteAllFromArrayTable(int e, int[][][] table) {
	
	int eIndex;

	for (int r = 0; r < table.length; r++) {
	    for (int c = 0; c < table[r].length; c++) {
		
		eIndex = -1;
		
		if (table[r][c] != null) {
		    // Update entries
		    for (int i = 0; i < table[r][c].length; i++) {
			if (table[r][c][i] > e) {
			    table[r][c][i] = table[r][c][i] - 1; // Decrement larger entries
			} else if (table[r][c][i] == e) {
			    eIndex = i; // Note position of e entry
			}
		    }
		    
		    // Remove e if it was found amongst entries
		    if (eIndex >= 0) {
			table[r][c] = deleteFromArray(eIndex, table[r][c]);
		    }
		} 
	    } 
	}

	return table;
    }

    public static boolean isArrayTableEntry(int row, int col, int entry, int[][][] table) {
	
	if (table[row][col] != null) {
	    for (int i = 0; i < table[row][col].length; i++) {
		if (table[row][col][i] == entry) {
		    return true;
		}
	    }
	}

	return false;
    }


    public static int[][][] addArrayTableEntry(int row, int col, int entry, int[][][] table) {

	if (table[row][col] != null) {
	    table[row][col] = addToArray(entry, table[row][col]);
	} else {
	    table[row][col] = new int[1];
	    table[row][col][0] = entry;
	}
	return table;
    }


    public static int[][][] deleteArrayTableEntry(int row, int col, int entry, int[][][] table) {

	int eIndex = -1;
	
	for (int i = 0; i < table[row][col].length; i++) {
	    if (table[row][col][i] == entry) {
		eIndex = i;
		break;
	    }
	}
	
	if (eIndex >= 0) {
	    table[row][col] = deleteFromArray(eIndex, table[row][col]);
	}
	return table;
    }

    public static int percent(boolean[] b) {

	int r = 0;
	    
	for (int i = 0; i < b.length; i++) {
	    if (b[i]) {
		r++;
	    }
	}
	    
	float r2 = (float) r;
	float t = (float) b.length;
	int p = 100;
	
	if (t > 0) {
	    float p2 = 100 * (r2 / t);
	    p = (int) p2;
	}

	return p;
    }



    public static int[] cropArray(int[] array, int length) {

	if (length == 0) {
	    return null;
	} else {
	    int[] cropped = new int[length];
	    System.arraycopy(array, 0, cropped, 0, length);
	    return cropped;
	} 
    }

    public static boolean[] toPredicate(int[] set, int size) {
	
	if (set != null) {
	    boolean[] pred = new boolean[size];
	    
	    for (int i = 0; i < set.length; i++) {
		pred[set[i]] = true;
	    }
	    return pred;

	} else {
	    return null;
	}
    }

    public static int[] toSet(boolean[] pred) {

	if (pred != null) {
	    int[] set = new int[pred.length];
	    int len = 0;
	    
	    for (int i = 0; i < pred.length; i++) {
		if (pred[i]) {
		    set[len] = i;
		    len++;
		}
	    }
	    
	    return cropArray(set, len);
	} else {
	    return null;
	}
    }



    public static boolean[] column(boolean[][] table, int c) {

	boolean[] col = new boolean[table.length];
	
	for (int i = 0; i < table.length; i++) {
	    col[i] = table[i][c];
	}

	return col;
    }

    public static int[] trueInRow(int row, boolean[][] table) {
	int[] tir = new int[table[row].length];
	int index = 0;

	for (int col = 0; col < table[row].length; col++) {
	    if (table[row][col]) {
		tir[index] = col;
		index++;
	    }
	}

	return ArrayUtils.cropArray(tir, index);
    }

    public static int[] trueInColumn(int col, boolean[][] table) {
	int[] tic = new int[table.length];
	int index = 0;

	for (int row = 0; row < table.length; row++) {
	    if (table[row][col]) {
		tic[index] = row;
		index++;
	    }
	}

	return cropArray(tic, index);
    }


    /*
     * Set operations on arrays
     */
    public static int[] intersection(int[] a, int[] b) {

	if (a == null || b == null) {
	    return null;

	} else {
	    int[] c = new int[a.length];
	    int len = 0;
	    
	    for (int i = 0; i < a.length; i++) {
		if (inArray(a[i], b)) {
		    c[len] = a[i];
		    len++;
		}
	    }
	    
	    return cropArray(c, len);
	}
    }


    /*
     * Element-wise boolean operations on arrays
     */

    public static boolean[] andArray(boolean[] a, boolean[] b) {
	
	boolean[] c;
	boolean alonger = a.length > b.length;

	if (alonger) {
	    c = new boolean[a.length];
	} else {
	    c = new boolean[b.length];
	}
	
	for (int i = 0; i < c.length; i++) {

	    if (i < a.length && i < b.length) {
		c[i] = a[i] && b[i];
	    } else if (alonger) {
		c[i] = a[i];
	    } else {
		c[i] = b[i];
	    }
	}

	return c;
    }

}

